"""哈希、Base64、对称/非对称加解密（cryptography）、Unix 账户查询（仅 POSIX）。"""

from __future__ import annotations

import base64
import binascii
import hashlib
import hmac
import os
import secrets
from typing import Any

MAX_TEXT = 512_000
MAX_HEX_LEN = 2_000_000  # ~1MB binary
MAX_B64_LEN = 2_000_000

try:
    from cryptography.hazmat.primitives import hashes, serialization
    from cryptography.hazmat.primitives.asymmetric import ec, padding, rsa
    from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
    from cryptography.hazmat.primitives.ciphers.aead import AESGCM
    from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
    from cryptography.hazmat.backends import default_backend

    _CRYPTO_OK = True
    _CRYPTO_ERR: str | None = None
except ImportError as e:  # pragma: no cover
    _CRYPTO_OK = False
    _CRYPTO_ERR = str(e)


def _need_crypto() -> dict[str, Any] | None:
    if not _CRYPTO_OK:
        return {
            "ok": False,
            "error": f"cryptography not available: {_CRYPTO_ERR}; pip install cryptography",
        }
    return None


def _bytes_from_text_or_hex(payload: dict[str, Any], key: str = "text") -> tuple[bytes | None, dict[str, Any] | None]:
    hx = payload.get("hex")
    if hx is not None:
        s = str(hx).strip()
        if len(s) > MAX_HEX_LEN:
            return None, {"ok": False, "error": "hex input too large"}
        try:
            return binascii.unhexlify(s.replace(" ", "")), None
        except binascii.Error as e:
            return None, {"ok": False, "error": f"invalid hex: {e}"}
    t = str(payload.get(key) or "")
    if len(t) > MAX_TEXT:
        return None, {"ok": False, "error": f"{key} exceeds {MAX_TEXT} chars"}
    return t.encode("utf-8"), None


def op_hash(algo: str, payload: dict[str, Any]) -> dict[str, Any]:
    algo = algo.lower().replace("-", "")
    data, err = _bytes_from_text_or_hex(payload)
    if err:
        return err
    assert data is not None
    if algo == "md5":
        d = hashlib.md5(data, usedforsecurity=False).digest()
    elif algo == "sha1":
        d = hashlib.sha1(data, usedforsecurity=False).digest()
    elif algo == "sha256":
        d = hashlib.sha256(data).digest()
    elif algo == "sha512":
        d = hashlib.sha512(data).digest()
    else:
        return {"ok": False, "error": "algorithm must be md5|sha1|sha256|sha512"}
    out_hex = d.hex()
    return {"ok": True, "algorithm": algo, "hex": out_hex, "bytes": len(d)}


def op_hmac_sha256(payload: dict[str, Any]) -> dict[str, Any]:
    msg, err = _bytes_from_text_or_hex(payload, "message")
    if err:
        return err
    assert msg is not None
    key_raw = payload.get("key_hex")
    if key_raw is not None:
        try:
            key = binascii.unhexlify(str(key_raw).replace(" ", ""))
        except binascii.Error as e:
            return {"ok": False, "error": f"key_hex: {e}"}
    else:
        kt = str(payload.get("key_text") or "")
        key = kt.encode("utf-8")
    if not key:
        return {"ok": False, "error": "need key_hex or key_text"}
    d = hmac.new(key, msg, hashlib.sha256).digest()
    return {"ok": True, "hex": d.hex()}


def op_base64_encode(payload: dict[str, Any]) -> dict[str, Any]:
    data, err = _bytes_from_text_or_hex(payload)
    if err:
        return err
    assert data is not None
    b = base64.standard_b64encode(data).decode("ascii")
    return {"ok": True, "base64": b}


def op_base64_decode(payload: dict[str, Any]) -> dict[str, Any]:
    s = str(payload.get("base64") or "").strip()
    if len(s) > MAX_B64_LEN:
        return {"ok": False, "error": "base64 input too large"}
    try:
        raw = base64.standard_b64decode(s)
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"decode failed: {e}"}
    try:
        text = raw.decode("utf-8")
        return {"ok": True, "text": text, "bytes": len(raw)}
    except UnicodeDecodeError:
        return {"ok": True, "hex": raw.hex(), "bytes": len(raw), "note": "not valid utf-8; hex returned"}


def op_random_hex(payload: dict[str, Any]) -> dict[str, Any]:
    try:
        n = int(payload.get("byte_length", 16))
    except (TypeError, ValueError):
        return {"ok": False, "error": "byte_length must be int"}
    n = max(1, min(256, n))
    return {"ok": True, "hex": secrets.token_hex(n)}


def op_pbkdf2_hmac_sha256(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    pw = str(payload.get("password") or "").encode("utf-8")
    salt_hex = str(payload.get("salt_hex") or "")
    try:
        salt = binascii.unhexlify(salt_hex.replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"salt_hex: {e}"}
    try:
        it = int(payload.get("iterations", 390_000))
    except (TypeError, ValueError):
        return {"ok": False, "error": "iterations must be int"}
    it = max(10_000, min(5_000_000, it))
    try:
        length = int(payload.get("length_bytes", 32))
    except (TypeError, ValueError):
        return {"ok": False, "error": "length_bytes must be int"}
    length = max(16, min(64, length))
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=length,
        salt=salt,
        iterations=it,
        backend=default_backend(),
    )
    dk = kdf.derive(pw)
    return {"ok": True, "hex": dk.hex(), "iterations": it, "length_bytes": length}


def _pkcs7_pad(data: bytes, block: int) -> bytes:
    p = block - (len(data) % block)
    return data + bytes([p]) * p


def _pkcs7_unpad(data: bytes) -> bytes:
    if not data:
        raise ValueError("empty")
    p = data[-1]
    if p < 1 or p > 16:
        raise ValueError("bad padding")
    if data[-p:] != bytes([p]) * p:
        raise ValueError("bad padding")
    return data[:-p]


def op_aes_gcm_encrypt(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    key_hex = str(payload.get("key_hex") or "")
    try:
        key = binascii.unhexlify(key_hex.replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"key_hex: {e}"}
    if len(key) not in (16, 24, 32):
        return {"ok": False, "error": "AES key must be 16/24/32 bytes (32/48/64 hex chars)"}
    pt, e2 = _bytes_from_text_or_hex(payload)
    if e2:
        return e2
    assert pt is not None
    nonce_hex = str(payload.get("nonce_hex") or "").strip()
    if nonce_hex:
        try:
            nonce = binascii.unhexlify(nonce_hex.replace(" ", ""))
        except binascii.Error as e:
            return {"ok": False, "error": f"nonce_hex: {e}"}
        if len(nonce) != 12:
            return {"ok": False, "error": "GCM nonce must be 12 bytes"}
    else:
        nonce = secrets.token_bytes(12)
    aesgcm = AESGCM(key)
    ad = str(payload.get("associated_data") or "").encode("utf-8")
    ct = aesgcm.encrypt(nonce, pt, ad if ad else None)
    return {
        "ok": True,
        "nonce_hex": nonce.hex(),
        "ciphertext_hex": ct.hex(),
        "associated_data_was_set": bool(ad),
    }


def op_aes_gcm_decrypt(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    key_hex = str(payload.get("key_hex") or "")
    try:
        key = binascii.unhexlify(key_hex.replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"key_hex: {e}"}
    if len(key) not in (16, 24, 32):
        return {"ok": False, "error": "AES key length invalid"}
    try:
        nonce = binascii.unhexlify(str(payload.get("nonce_hex") or "").replace(" ", ""))
        ct = binascii.unhexlify(str(payload.get("ciphertext_hex") or "").replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"hex: {e}"}
    aesgcm = AESGCM(key)
    ad = str(payload.get("associated_data") or "").encode("utf-8")
    try:
        pt = aesgcm.decrypt(nonce, ct, ad if ad else None)
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"decrypt failed: {e}"}
    try:
        return {"ok": True, "text": pt.decode("utf-8")}
    except UnicodeDecodeError:
        return {"ok": True, "hex": pt.hex(), "note": "binary plaintext as hex"}


def op_aes_cbc_encrypt(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    key_hex = str(payload.get("key_hex") or "")
    iv_hex = str(payload.get("iv_hex") or "")
    try:
        key = binascii.unhexlify(key_hex.replace(" ", ""))
        iv = binascii.unhexlify(iv_hex.replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"hex: {e}"}
    if len(key) not in (16, 24, 32) or len(iv) != 16:
        return {"ok": False, "error": "AES CBC needs 16-byte IV and key 16/24/32 bytes"}
    pt, e2 = _bytes_from_text_or_hex(payload)
    if e2:
        return e2
    assert pt is not None
    padded = _pkcs7_pad(pt, 16)
    cipher = Cipher(algorithms.AES(key), modes.CBC(iv), backend=default_backend())
    enc = cipher.encryptor()
    ct = enc.update(padded) + enc.finalize()
    return {"ok": True, "ciphertext_hex": ct.hex()}


def op_aes_cbc_decrypt(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    key_hex = str(payload.get("key_hex") or "")
    iv_hex = str(payload.get("iv_hex") or "")
    try:
        key = binascii.unhexlify(key_hex.replace(" ", ""))
        iv = binascii.unhexlify(iv_hex.replace(" ", ""))
        ct = binascii.unhexlify(str(payload.get("ciphertext_hex") or "").replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"hex: {e}"}
    if len(iv) != 16 or len(key) not in (16, 24, 32):
        return {"ok": False, "error": "invalid key/iv length"}
    cipher = Cipher(algorithms.AES(key), modes.CBC(iv), backend=default_backend())
    dec = cipher.decryptor()
    try:
        padded = dec.update(ct) + dec.finalize()
        pt = _pkcs7_unpad(padded)
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"decrypt: {e}"}
    try:
        return {"ok": True, "text": pt.decode("utf-8")}
    except UnicodeDecodeError:
        return {"ok": True, "hex": pt.hex()}


def op_des_cbc_encrypt(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    try:
        key = binascii.unhexlify(str(payload.get("key_hex") or "").replace(" ", ""))
        iv = binascii.unhexlify(str(payload.get("iv_hex") or "").replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"hex: {e}"}
    if len(key) != 8 or len(iv) != 8:
        return {"ok": False, "error": "DES needs 8-byte key and 8-byte IV (16 hex each)"}
    pt, e2 = _bytes_from_text_or_hex(payload)
    if e2:
        return e2
    assert pt is not None
    padded = _pkcs7_pad(pt, 8)
    cipher = Cipher(algorithms.DES(key), modes.CBC(iv), backend=default_backend())
    enc = cipher.encryptor()
    ct = enc.update(padded) + enc.finalize()
    return {
        "ok": True,
        "ciphertext_hex": ct.hex(),
        "note": "single DES is weak; prefer AES-GCM",
    }


def op_des_cbc_decrypt(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    try:
        key = binascii.unhexlify(str(payload.get("key_hex") or "").replace(" ", ""))
        iv = binascii.unhexlify(str(payload.get("iv_hex") or "").replace(" ", ""))
        ct = binascii.unhexlify(str(payload.get("ciphertext_hex") or "").replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"hex: {e}"}
    if len(key) != 8 or len(iv) != 8:
        return {"ok": False, "error": "DES needs 8-byte key and IV"}
    cipher = Cipher(algorithms.DES(key), modes.CBC(iv), backend=default_backend())
    dec = cipher.decryptor()
    try:
        padded = dec.update(ct) + dec.finalize()
        pt = _pkcs7_unpad(padded)
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"decrypt: {e}"}
    try:
        return {"ok": True, "text": pt.decode("utf-8")}
    except UnicodeDecodeError:
        return {"ok": True, "hex": pt.hex()}


def op_rsa_generate(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    try:
        bits = int(payload.get("bits", 2048))
    except (TypeError, ValueError):
        return {"ok": False, "error": "bits must be int"}
    bits = max(1024, min(4096, bits))
    sk = rsa.generate_private_key(
        public_exponent=65537, key_size=bits, backend=default_backend()
    )
    pk = sk.public_key()
    priv_pem = sk.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    ).decode("ascii")
    pub_pem = pk.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo,
    ).decode("ascii")
    return {"ok": True, "bits": bits, "private_pem": priv_pem, "public_pem": pub_pem}


def op_rsa_encrypt(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    pem = str(payload.get("public_pem") or "")
    data, e2 = _bytes_from_text_or_hex(payload)
    if e2:
        return e2
    assert data is not None
    try:
        pub = serialization.load_pem_public_key(pem.encode("ascii"), backend=default_backend())
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"PEM public key: {e}"}
    if not isinstance(pub, rsa.RSAPublicKey):
        return {"ok": False, "error": "not an RSA public key"}
    max_len = (pub.key_size // 8) - 2 * 32 - 2
    if len(data) > max_len:
        return {
            "ok": False,
            "error": f"plaintext too long for OAEP-SHA256 (max {max_len} bytes); use AES hybrid",
        }
    ct = pub.encrypt(
        data,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None,
        ),
    )
    return {"ok": True, "ciphertext_hex": ct.hex()}


def op_rsa_decrypt(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    pem = str(payload.get("private_pem") or "")
    try:
        ct = binascii.unhexlify(str(payload.get("ciphertext_hex") or "").replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"ciphertext_hex: {e}"}
    try:
        sk = serialization.load_pem_private_key(
            pem.encode("ascii"), password=None, backend=default_backend()
        )
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"PEM private key: {e}"}
    if not isinstance(sk, rsa.RSAPrivateKey):
        return {"ok": False, "error": "not an RSA private key"}
    try:
        pt = sk.decrypt(
            ct,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None,
            ),
        )
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"decrypt: {e}"}
    try:
        return {"ok": True, "text": pt.decode("utf-8")}
    except UnicodeDecodeError:
        return {"ok": True, "hex": pt.hex()}


def op_ecc_generate(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    curve_name = str(payload.get("curve", "secp256r1")).lower()
    curve_map = {
        "secp256r1": ec.SECP256R1(),
        "prime256v1": ec.SECP256R1(),
        "secp384r1": ec.SECP384R1(),
        "secp521r1": ec.SECP521R1(),
    }
    curve = curve_map.get(curve_name)
    if curve is None:
        return {"ok": False, "error": "curve must be secp256r1|secp384r1|secp521r1"}
    sk = ec.generate_private_key(curve, default_backend())
    pk = sk.public_key()
    priv_pem = sk.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    ).decode("ascii")
    pub_pem = pk.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo,
    ).decode("ascii")
    return {"ok": True, "curve": curve_name, "private_pem": priv_pem, "public_pem": pub_pem}


def op_ecdsa_sign(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    pem = str(payload.get("private_pem") or "")
    msg, e2 = _bytes_from_text_or_hex(payload, "message")
    if e2:
        return e2
    assert msg is not None
    try:
        sk = serialization.load_pem_private_key(
            pem.encode("ascii"), password=None, backend=default_backend()
        )
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"PEM: {e}"}
    if not isinstance(sk, ec.EllipticCurvePrivateKey):
        return {"ok": False, "error": "not an EC private key"}
    sig = sk.sign(msg, ec.ECDSA(hashes.SHA256()))
    return {"ok": True, "signature_hex": sig.hex()}


def op_ecdsa_verify(payload: dict[str, Any]) -> dict[str, Any]:
    err = _need_crypto()
    if err:
        return err
    pem = str(payload.get("public_pem") or "")
    msg, e2 = _bytes_from_text_or_hex(payload, "message")
    if e2:
        return e2
    assert msg is not None
    try:
        sig = binascii.unhexlify(str(payload.get("signature_hex") or "").replace(" ", ""))
    except binascii.Error as e:
        return {"ok": False, "error": f"signature_hex: {e}"}
    try:
        pk = serialization.load_pem_public_key(pem.encode("ascii"), backend=default_backend())
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": f"PEM: {e}"}
    if not isinstance(pk, ec.EllipticCurvePublicKey):
        return {"ok": False, "error": "not an EC public key"}
    try:
        pk.verify(sig, msg, ec.ECDSA(hashes.SHA256()))
    except Exception:  # noqa: BLE001
        return {"ok": True, "valid": False}
    return {"ok": True, "valid": True}


def op_unix_passwd_list(payload: dict[str, Any]) -> dict[str, Any]:
    if os.name == "nt":
        return {"ok": False, "error": "Unix only: pwd module not available on Windows"}
    try:
        import pwd
    except ImportError:
        return {"ok": False, "error": "pwd module unavailable"}
    try:
        lim = int(payload.get("limit", 200))
    except (TypeError, ValueError):
        lim = 200
    lim = max(1, min(2000, lim))
    out: list[dict[str, Any]] = []
    for i, p in enumerate(pwd.getpwall()):
        if i >= lim:
            break
        out.append(
            {
                "name": p.pw_name,
                "uid": p.pw_uid,
                "gid": p.pw_gid,
                "dir": p.pw_dir,
                "shell": p.pw_shell,
                "gecos": p.pw_gecos,
            }
        )
    return {"ok": True, "entries": out, "truncated": len(out) >= lim, "note": "no password field"}


def op_unix_group_list(payload: dict[str, Any]) -> dict[str, Any]:
    if os.name == "nt":
        return {"ok": False, "error": "Unix only"}
    try:
        import grp
    except ImportError:
        return {"ok": False, "error": "grp module unavailable"}
    try:
        lim = int(payload.get("limit", 200))
    except (TypeError, ValueError):
        lim = 200
    lim = max(1, min(2000, lim))
    out: list[dict[str, Any]] = []
    for i, g in enumerate(grp.getgrall()):
        if i >= lim:
            break
        out.append(
            {
                "name": g.gr_name,
                "gid": g.gr_gid,
                "members": list(g.gr_mem),
            }
        )
    return {"ok": True, "entries": out, "truncated": len(out) >= lim}


def op_unix_getpwnam(payload: dict[str, Any]) -> dict[str, Any]:
    if os.name == "nt":
        return {"ok": False, "error": "Unix only"}
    import pwd

    name = str(payload.get("name") or "")
    if not name:
        return {"ok": False, "error": "name required"}
    try:
        p = pwd.getpwnam(name)
    except KeyError:
        return {"ok": False, "error": "not found"}
    return {
        "ok": True,
        "entry": {
            "name": p.pw_name,
            "uid": p.pw_uid,
            "gid": p.pw_gid,
            "dir": p.pw_dir,
            "shell": p.pw_shell,
            "gecos": p.pw_gecos,
        },
    }


def op_unix_getgrnam(payload: dict[str, Any]) -> dict[str, Any]:
    if os.name == "nt":
        return {"ok": False, "error": "Unix only"}
    import grp

    name = str(payload.get("name") or "")
    if not name:
        return {"ok": False, "error": "name required"}
    try:
        g = grp.getgrnam(name)
    except KeyError:
        return {"ok": False, "error": "not found"}
    return {
        "ok": True,
        "entry": {
            "name": g.gr_name,
            "gid": g.gr_gid,
            "members": list(g.gr_mem),
        },
    }


def run_operation(operation: str, payload: dict[str, Any]) -> dict[str, Any]:
    op = operation.strip().lower().replace("-", "_")
    if op in ("hash_md5", "md5"):
        return op_hash("md5", payload)
    if op in ("hash_sha1", "sha1"):
        return op_hash("sha1", payload)
    if op in ("hash_sha256", "sha256"):
        return op_hash("sha256", payload)
    if op in ("hash_sha512", "sha512"):
        return op_hash("sha512", payload)
    if op == "hash":
        algo = str(payload.get("algorithm") or "sha256").lower()
        return op_hash(algo, payload)
    if op in ("hmac_sha256",):
        return op_hmac_sha256(payload)
    if op in ("base64_encode", "b64encode"):
        return op_base64_encode(payload)
    if op in ("base64_decode", "b64decode"):
        return op_base64_decode(payload)
    if op in ("random_hex", "random_bytes_hex"):
        return op_random_hex(payload)
    if op in ("pbkdf2_hmac_sha256", "pbkdf2"):
        return op_pbkdf2_hmac_sha256(payload)
    if op in ("aes_gcm_encrypt",):
        return op_aes_gcm_encrypt(payload)
    if op in ("aes_gcm_decrypt",):
        return op_aes_gcm_decrypt(payload)
    if op in ("aes_cbc_encrypt",):
        return op_aes_cbc_encrypt(payload)
    if op in ("aes_cbc_decrypt",):
        return op_aes_cbc_decrypt(payload)
    if op in ("des_cbc_encrypt", "des_encrypt"):
        return op_des_cbc_encrypt(payload)
    if op in ("des_cbc_decrypt", "des_decrypt"):
        return op_des_cbc_decrypt(payload)
    if op in ("rsa_generate", "rsa_keygen"):
        return op_rsa_generate(payload)
    if op in ("rsa_encrypt", "rsa_oaep_encrypt"):
        return op_rsa_encrypt(payload)
    if op in ("rsa_decrypt", "rsa_oaep_decrypt"):
        return op_rsa_decrypt(payload)
    if op in ("ecc_generate", "ec_keygen", "ecc_keygen"):
        return op_ecc_generate(payload)
    if op in ("ecdsa_sign", "ecc_sign"):
        return op_ecdsa_sign(payload)
    if op in ("ecdsa_verify", "ecc_verify"):
        return op_ecdsa_verify(payload)
    if op in ("unix_passwd_list", "passwd_list"):
        return op_unix_passwd_list(payload)
    if op in ("unix_group_list", "group_list"):
        return op_unix_group_list(payload)
    if op in ("unix_getpwnam", "getpwnam"):
        return op_unix_getpwnam(payload)
    if op in ("unix_getgrnam", "getgrnam"):
        return op_unix_getgrnam(payload)
    return {
        "ok": False,
        "error": f"unknown operation: {operation!r}",
        "hint": "hash|hmac_sha256|base64_encode|base64_decode|random_hex|pbkdf2_hmac_sha256|"
        "aes_gcm_encrypt|aes_gcm_decrypt|aes_cbc_encrypt|aes_cbc_decrypt|"
        "des_cbc_encrypt|des_cbc_decrypt|rsa_generate|rsa_encrypt|rsa_decrypt|"
        "ecc_generate|ecdsa_sign|ecdsa_verify|unix_passwd_list|unix_group_list|unix_getpwnam|unix_getgrnam",
    }



"""kit crypto_codec / format_data 工具逻辑。"""

from __future__ import annotations

from aicd.kit import crypto_codec, format_data


def test_hash_sha256() -> None:
    r = crypto_codec.run_operation("hash_sha256", {"text": "abc"})
    assert r["ok"]
    assert r["hex"] == "ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad"


def test_base64_roundtrip() -> None:
    e = crypto_codec.run_operation("base64_encode", {"text": "hi"})
    assert e["ok"]
    d = crypto_codec.run_operation("base64_decode", {"base64": e["base64"]})
    assert d["ok"]
    assert d["text"] == "hi"


def test_aes_gcm_roundtrip() -> None:
    key_hex = "00" * 32
    enc = crypto_codec.run_operation(
        "aes_gcm_encrypt",
        {"key_hex": key_hex, "text": "secret", "nonce_hex": "01" * 12},
    )
    assert enc["ok"]
    dec = crypto_codec.run_operation(
        "aes_gcm_decrypt",
        {
            "key_hex": key_hex,
            "nonce_hex": enc["nonce_hex"],
            "ciphertext_hex": enc["ciphertext_hex"],
        },
    )
    assert dec["ok"]
    assert dec["text"] == "secret"


def test_rsa_roundtrip() -> None:
    gen = crypto_codec.run_operation("rsa_generate", {"bits": 2048})
    assert gen["ok"]
    pub = gen["public_pem"]
    priv = gen["private_pem"]
    enc = crypto_codec.run_operation(
        "rsa_encrypt", {"public_pem": pub, "text": "hello-rsa"}
    )
    assert enc["ok"]
    dec = crypto_codec.run_operation(
        "rsa_decrypt",
        {"private_pem": priv, "ciphertext_hex": enc["ciphertext_hex"]},
    )
    assert dec["ok"]
    assert dec["text"] == "hello-rsa"


def test_ecdsa_sign_verify() -> None:
    gen = crypto_codec.run_operation("ecc_generate", {"curve": "secp256r1"})
    assert gen["ok"]
    sig = crypto_codec.run_operation(
        "ecdsa_sign",
        {"private_pem": gen["private_pem"], "message": "payload"},
    )
    assert sig["ok"]
    v = crypto_codec.run_operation(
        "ecdsa_verify",
        {
            "public_pem": gen["public_pem"],
            "message": "payload",
            "signature_hex": sig["signature_hex"],
        },
    )
    assert v["ok"] and v["valid"]


def test_json_path_and_html() -> None:
    j = format_data.run_operation(
        "json_path_get", {"text": '{"a":{"b":[1,{"x":9}]}}', "path": "a.b.1.x"}
    )
    assert j["ok"]
    assert j["value"] == 9
    h = format_data.run_operation(
        "html_to_text", {"html": "<p>Hello &amp; <b>world</b></p>"}
    )
    assert h["ok"]
    assert "Hello" in h["text"] and "world" in h["text"]


def test_xml_find() -> None:
    x = format_data.run_operation(
        "xml_find_tags",
        {"xml": '<r><item id="1">a</item><item>b</item></r>', "tag": "item"},
    )
    assert x["ok"]
    assert x["count"] == 2


def test_structure_transpose() -> None:
    r = format_data.run_operation(
        "structure_convert",
        {
            "mode": "transpose_records",
            "value": [{"a": 1, "b": 2}, {"a": 3, "b": 4}],
        },
    )
    assert r["ok"]
    assert r["series"]["a"] == [1, 3]



from __future__ import annotations

import os

from aicd.cli_encoding import configure_cli_utf8, subprocess_env_utf8


def test_subprocess_env_utf8_sets_defaults() -> None:
    e = subprocess_env_utf8({"CUSTOM": "1"})
    assert e.get("PYTHONIOENCODING") == "utf-8"
    assert e.get("PYTHONUTF8") == "1"
    assert e.get("CUSTOM") == "1"


def test_subprocess_env_does_not_override_explicit() -> None:
    e = subprocess_env_utf8({"PYTHONIOENCODING": "latin-1"})
    assert e.get("PYTHONIOENCODING") == "latin-1"


def test_configure_cli_utf8_when_unset(monkeypatch) -> None:
    monkeypatch.delenv("PYTHONIOENCODING", raising=False)
    monkeypatch.delenv("PYTHONUTF8", raising=False)
    configure_cli_utf8()
    configure_cli_utf8()
    assert os.environ.get("PYTHONIOENCODING") == "utf-8"
    assert os.environ.get("PYTHONUTF8") == "1"



"""只读聊天历史 HTTP API（可选依赖 fastapi/uvicorn）。"""

from __future__ import annotations

import os
from contextlib import asynccontextmanager
from pathlib import Path
from typing import Any, AsyncGenerator

from aicd.db.store import TaskStore
from aicd.home import init_home


def create_chat_read_api(*, db_path: Path, api_key: str | None) -> Any:
    from fastapi import Depends, FastAPI, Header, HTTPException, Query

    @asynccontextmanager
    async def lifespan(app: FastAPI) -> AsyncGenerator[None, None]:
        store = TaskStore(db_path)
        conn = await store.connect()
        app.state.store = store
        app.state.conn = conn
        yield
        await conn.close()

    app = FastAPI(title="Aicd Chat Read API", version="1.0", lifespan=lifespan)

    async def optional_auth(
        x_api_key: str | None = Header(default=None, alias="X-API-Key"),
        authorization: str | None = Header(default=None),
    ) -> None:
        expected = (api_key or os.environ.get("AICD_API_KEY") or "").strip()
        if not expected:
            return
        got = (x_api_key or "").strip()
        if not got and authorization:
            auth = authorization.strip()
            if auth.lower().startswith("bearer "):
                got = auth[7:].strip()
        if got != expected:
            raise HTTPException(status_code=401, detail="invalid or missing API key")

    def _msg_row(r: Any) -> dict[str, Any]:
        return {
            "id": r.id,
            "session_id": r.session_id,
            "role": r.role,
            "content": r.content,
            "tool_calls_json": r.tool_calls_json,
            "created_at": r.created_at,
            "tool_call_id": r.tool_call_id,
            "tool_name": r.tool_name,
        }

    def _sum_row(r: Any) -> dict[str, Any]:
        return {
            "id": r.id,
            "session_id": r.session_id,
            "content": r.content,
            "covers_up_to_message_id": r.covers_up_to_message_id,
            "created_at": r.created_at,
        }

    @app.get("/health")
    async def health() -> dict[str, bool]:
        return {"ok": True}

    @app.get("/v1/sessions/{session_id}/messages")
    async def list_messages(
        session_id: str,
        limit: int = Query(default=200, ge=1, le=2000),
        offset: int = Query(default=0, ge=0),
        _: None = Depends(optional_auth),
    ) -> dict[str, Any]:
        store: TaskStore = app.state.store
        conn = app.state.conn
        rows = await store.list_chat_messages(
            conn, session_id, limit=limit, offset=offset
        )
        total = await store.count_chat_messages(conn, session_id)
        return {
            "session_id": session_id,
            "total": total,
            "offset": offset,
            "limit": limit,
            "messages": [_msg_row(r) for r in rows],
        }

    @app.get("/v1/sessions/{session_id}/summaries")
    async def list_summaries(
        session_id: str,
        limit: int = Query(default=50, ge=1, le=500),
        _: None = Depends(optional_auth),
    ) -> dict[str, Any]:
        store: TaskStore = app.state.store
        conn = app.state.conn
        rows = await store.list_chat_summaries(conn, session_id, limit=limit)
        return {
            "session_id": session_id,
            "summaries": [_sum_row(r) for r in rows],
        }

    return app


async def run_serve(
    *,
    home: Path | None,
    host: str,
    port: int,
    api_key: str | None,
) -> None:
    import uvicorn

    _root, layout, _cfg = init_home(home)
    db = layout["db"]
    app = create_chat_read_api(db_path=db, api_key=api_key)
    config = uvicorn.Config(app, host=host, port=port, log_level="info")
    await uvicorn.Server(config).serve()



"""docx_compose：表格与东亚字体默认值。"""

from __future__ import annotations

from pathlib import Path

from docx import Document

from aicd.tools.docx_compose import docx_compose


def _policy(p: Path) -> None:
    return None


def test_docx_table_rows_as_json_string(tmp_path: Path) -> None:
    """部分模型把 table.rows 序列化成 JSON 字符串。"""
    p = tmp_path / "out_json_rows.docx"
    r = docx_compose(
        p,
        [
            {
                "type": "table",
                "rows": '[["A","B"],["1","2"]]',
            },
        ],
        policy_check=_policy,
    )
    assert r["ok"] is True
    assert r["tables_added"] == 1


def test_docx_table_and_simsun_defaults(tmp_path: Path) -> None:
    p = tmp_path / "out.docx"
    r = docx_compose(
        p,
        [
            {"type": "heading", "text": "H", "level": 1},
            {"type": "paragraph", "text": "body"},
            {
                "type": "table",
                "rows": [["A", "B"], ["1", "2"]],
                "header_row": True,
            },
        ],
        policy_check=_policy,
        default_east_asia_font="宋体",
        default_color_hex="000000",
        default_body_font_pt=11.0,
    )
    assert r["ok"] is True
    assert r["tables_added"] == 1
    assert r["defaults_applied"] is True
    doc = Document(str(p))
    assert len(doc.tables) == 1
    assert len(doc.paragraphs) >= 2



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from aicd.agent.tool_args_coerce import coerce_tool_json_array, coerce_tool_json_object
from aicd.tools.image_crop import crop_raster_image
from aicd.tools.image_transform import list_image_operations, operations_help, run_image_transform

if TYPE_CHECKING:
    from aicd.agent.tool_router import ToolRouter


def _resolve_image_params_paths(router: ToolRouter, params: dict[str, Any]) -> tuple[dict[str, Any], str | None]:
    q = dict(params)
    for key in ("overlay_path", "second_path"):
        v = q.get(key)
        if v:
            q[key] = str(router._resolve_path(str(v)))
    paths = q.get("paths")
    if isinstance(paths, str) and paths.strip():
        co = coerce_tool_json_array(paths, "params.paths", allow_empty=True)
        if isinstance(co, dict):
            return {}, str(co.get("error", "params.paths invalid"))
        paths = co
    if isinstance(paths, list):
        q["paths"] = [
            str(router._resolve_path(str(x))) for x in paths if str(x).strip()
        ]
    elif paths is not None and not isinstance(paths, list):
        return {}, "params.paths must be array or JSON array string"
    return q, None


def _image_transform(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    op = str(args.get("operation") or "").strip().lower().replace("-", "_")
    if op in ("help", "list_operations", "operations_help"):
        return {
            "ok": True,
            "operations_help": operations_help(),
            "known_operations": list_image_operations(),
        }
    outp_s = args.get("output_path")
    if not outp_s or not str(outp_s).strip():
        return {"ok": False, "error": "output_path is required"}
    out = router._resolve_path(str(outp_s).strip())
    inp_raw = args.get("input_path")
    if op == "solid_canvas" and (inp_raw is None or not str(inp_raw).strip()):
        inp = out.parent
    else:
        if inp_raw is None or not str(inp_raw).strip():
            return {
                "ok": False,
                "error": "input_path is required (optional only for solid_canvas)",
            }
        inp = router._resolve_path(str(inp_raw).strip())
    p_obj, p_err = coerce_tool_json_object(args.get("params"), "params")
    if p_err:
        return {"ok": False, "error": p_err}
    params, rp_err = _resolve_image_params_paths(router, p_obj)
    if rp_err:
        return {"ok": False, "error": rp_err}
    return run_image_transform(
        op,
        inp,
        out,
        params=params,
        policy_check=router._policy.check_allowed,
    )


def _image_crop(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    inp = router._resolve_path(str(args["input_path"]))
    out = router._resolve_path(str(args["output_path"]))
    brw = args.get("bbox_reference_width")
    brh = args.get("bbox_reference_height")
    bbox_rw = int(brw) if brw is not None and str(brw).strip() != "" else None
    bbox_rh = int(brh) if brh is not None and str(brh).strip() != "" else None
    if (bbox_rw is None) ^ (bbox_rh is None):
        return {
            "ok": False,
            "error": "bbox_reference_width and bbox_reference_height must be set together",
        }
    return crop_raster_image(
        inp,
        out,
        left=int(args["left"]),
        top=int(args["top"]),
        width=int(args["width"]),
        height=int(args["height"]),
        policy_check=router._policy.check_allowed,
        bbox_reference_width=bbox_rw,
        bbox_reference_height=bbox_rh,
    )


HANDLERS: dict[str, Any] = {
    "image_transform": _image_transform,
    "image_crop": _image_crop,
}



from __future__ import annotations

import pytest

from aicd.version import bump_version_string, parse_version


def test_parse() -> None:
    assert parse_version("0.0.1") == (0, 0, 1)


def test_bump_carry_minor() -> None:
    assert bump_version_string("0.0.1") == "0.0.2"
    assert bump_version_string("0.0.8") == "0.0.9"
    assert bump_version_string("0.0.9") == "0.1.0"
    assert bump_version_string("0.1.0") == "0.1.1"
    assert bump_version_string("0.3.9") == "0.4.0"


def test_bump_invalid() -> None:
    with pytest.raises(ValueError):
        parse_version("1.0")



from __future__ import annotations

from aicd.kit import time_log


def test_elapsed_ms() -> None:
    r = time_log.elapsed_ms("2020-01-01T00:00:00Z", "2020-01-01T00:00:01Z")
    assert r["ok"] is True
    assert r["duration_ms"] == 1000


def test_log_record_has_ts() -> None:
    r = time_log.log_record("info", "hi", x=1)
    assert r["ok"] is True
    assert r["record"]["level"] == "info"
    assert r["record"]["message"] == "hi"
    assert r["record"]["x"] == 1
    assert "ts" in r["record"]



from __future__ import annotations

import asyncio
import json
import os
import subprocess
import sys
from datetime import datetime, timezone
from pathlib import Path
from typing import Any, cast

from rich.text import Text
from textual import on
from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.containers import Horizontal, Vertical
from textual.widgets import DataTable, Footer, Header, RichLog

from aicd.env_probe import (
    BANNER_CRITICAL,
    BANNER_NEUTRAL,
    BANNER_RECOMMEND,
    startup_env_chat_banner_lines,
)
from aicd.runtime import RuntimeContext
from aicd.ui.slash_commands import (
    chat_slash_help_text,
    join_args_for_shell,
    parse_slash_line,
    slash_help_text,
    task_slash_help_text,
)
from aicd.agent.idle_pulse import is_main_idle_pulse
from aicd.ui.slash_suggester import ChatTextArea


def _try_write_system_clipboard(text: str) -> bool:
    """在不依赖 Textual 的情况下把纯文本写入 OS 剪贴板（TUI 内通常无法用鼠标拖选）。"""
    if sys.platform == "win32":
        clip = os.path.join(
            os.environ.get("SystemRoot", r"C:\Windows"), "System32", "clip.exe"
        )
        if not os.path.isfile(clip):
            return False
        try:
            cr = subprocess.run(
                [clip],
                input=text.encode("utf-16-le", errors="replace"),
                check=False,
                capture_output=True,
                creationflags=getattr(subprocess, "CREATE_NO_WINDOW", 0),
            )
            return cr.returncode == 0
        except OSError:
            return False
    if sys.platform == "darwin":
        try:
            cr = subprocess.run(
                ["pbcopy"],
                input=text.encode("utf-8", errors="replace"),
                check=False,
                capture_output=True,
            )
            return cr.returncode == 0
        except OSError:
            return False
    for argv in (["wl-copy"], ["xclip", "-selection", "clipboard"]):
        try:
            cr = subprocess.run(
                argv,
                input=text.encode("utf-8", errors="replace"),
                check=False,
                capture_output=True,
            )
            if cr.returncode == 0:
                return True
        except OSError:
            continue
    return False


class AicdTui(App[None]):
    """主界面；AI 对话进行中标题栏脉冲提示。"""

    TITLE = "Aicd"

    BINDINGS = [
        Binding(
            "ctrl+shift+c",
            "cancel_chat",
            "停止AI",
            tooltip="中断当前主对话轮次（等同 /chat stop、/task stop）；后台 task_ai/task_shell 不停",
            show=True,
            priority=True,
        ),
    ]

    CSS = """
    #left { width: 1fr; min-width: 40; }
    #right { width: 45; min-width: 36; }
    #chat { height: 1fr; }
    #msg {
        height: 8;
        min-height: 4;
        max-height: 18;
    }
    #tasklog { height: 1fr; }
    DataTable { height: 12; }
    Header#app-header.-ai-busy {
        background: $foreground 10%;
    }
    """

    def __init__(
        self,
        ctx: RuntimeContext,
        *,
        bootstrap_user_message: str | None = None,
    ) -> None:
        super().__init__()
        self.ctx = ctx
        self._bootstrap_user_message = (
            bootstrap_user_message.strip() if bootstrap_user_message else None
        )
        self._selected_task: str | None = None
        self._bus_task: asyncio.Task[None] | None = None
        self._shutdown_once = False
        self._ai_header_busy: bool = False
        self._ai_header_pulse: bool = False
        self._ai_activity_suffix: str = ""
        self._chat_q: asyncio.Queue[str] = asyncio.Queue()
        self._chat_pump_task: asyncio.Task[None] | None = None
        self._heartbeat_task: asyncio.Task[None] | None = None
        self._current_chat_turn: asyncio.Task[str] | None = None
        self._input_history: list[str] = []
        self._input_history_cap = 500
        # 与 _input_history_path 一致；用于跨重启恢复底部输入历史
        self._input_history_filename = "tui_input_history.jsonl"

    def compose(self) -> ComposeResult:
        yield Header(show_clock=True, id="app-header")
        with Horizontal():
            with Vertical(id="left"):
                yield RichLog(id="chat", wrap=True, highlight=True, markup=True)
                yield ChatTextArea(
                    placeholder=(
                        "Enter 发送；Shift+Enter 换行；/ 命令行可用 → 采纳幽灵补全"
                    ),
                    id="msg",
                    soft_wrap=True,
                    tab_behavior="indent",
                    show_line_numbers=False,
                )
            with Vertical(id="right"):
                yield DataTable(id="tasks", cursor_type="row", zebra_stripes=True)
                yield RichLog(id="tasklog", wrap=True, highlight=True, markup=True)
        yield Footer()

    async def on_mount(self) -> None:
        table = self.query_one("#tasks", DataTable)
        table.add_columns("id", "type", "status")
        table.cursor_type = "row"
        await self.refresh_task_table()
        self._bus_task = asyncio.create_task(self._bus_loop())
        self._chat_pump_task = asyncio.create_task(self._chat_pump_loop())
        self._heartbeat_task = asyncio.create_task(self._idle_heartbeat_loop())
        chat = self.query_one("#chat", RichLog)
        chat.write(
            Text(
                "Aicd TUI — 左侧聊天，右侧任务与日志。"
                "  后台任务未结束时仍可继续发消息：主对话会排队依次处理；可同时存在多个运行中的子任务。"
                "  /? 命令；/work 工作目录；/doc AI 输出；/chat 主对话；/task 后台任务；/history；/llm /agent；/exit。"
                "  输入 `/` 后可用右方向键采纳幽灵补全；聊天区 **Enter** 发送，**Shift+Enter** 换行。"
                "  Ctrl+Shift+C、/chat stop、/task stop 中断当前主对话轮次（清排队）；Ctrl+Q 退出。",
                style="bold",
            )
        )
        snap = getattr(self.ctx, "runtime_env_snapshot", None) or {}
        has_key = bool(str(self.ctx.config.llm.api_key or "").strip())
        try:
            _banner_styles = {
                BANNER_CRITICAL: "bold red",
                BANNER_RECOMMEND: "green",
                BANNER_NEUTRAL: "cyan",
            }
            for line, kind in startup_env_chat_banner_lines(
                snap, has_llm_api_key=has_key
            ):
                chat.write(Text(line, style=_banner_styles.get(kind, "cyan")))
        except Exception as e:  # noqa: BLE001
            chat.write(
                Text(
                    f"[环境检查生成失败: {e}] 仍可使用；详情见 data/runtime_env.yaml",
                    style="yellow",
                )
            )
        self.set_interval(0.45, self._tick_ai_header_pulse)
        self._load_input_history_from_disk()
        if self._bootstrap_user_message:
            asyncio.create_task(self._run_bootstrap_user_message())

    async def _run_bootstrap_user_message(self) -> None:
        """启动后自动投递首条用户消息（与手动输入同路径，仍可继续交互）。"""
        await asyncio.sleep(0)
        text = self._bootstrap_user_message
        self._bootstrap_user_message = None
        if not text:
            return
        self._push_input_history(text)
        self.query_one("#chat", RichLog).write(Text(f"[你] {text}", style="green"))
        await self._chat_q.put(text)
        turn_busy = (
            self._current_chat_turn is not None
            and not self._current_chat_turn.done()
        )
        if turn_busy:
            nq = self._chat_q.qsize()
            suffix = f"（队列中约 {nq} 条）" if nq > 0 else ""
            self.query_one("#chat", RichLog).write(
                Text(
                    f"[系统] 主 AI 仍处理上一条；本条已排队，将依次执行{suffix}",
                    style="dim cyan",
                )
            )

    def _apply_ai_header_subtitle(self) -> None:
        """标题栏：脉冲圆点 + 当前阶段（等待模型 / 执行哪个工具），避免「像卡死无输出」。"""
        if not self._ai_header_busy:
            return
        dot = "●" if self._ai_header_pulse else "○"
        core = (self._ai_activity_suffix or "AI 处理中…").strip().replace("\n", " ")
        if len(core) > 96:
            core = core[:93] + "…"
        self.sub_title = f"{dot} {core}"

    def _tick_ai_header_pulse(self) -> None:
        if not self._ai_header_busy:
            return
        self._ai_header_pulse = not self._ai_header_pulse
        self._apply_ai_header_subtitle()

    def _set_ai_header_busy(self, busy: bool) -> None:
        self._ai_header_busy = busy
        if not busy:
            self._ai_header_pulse = False
            self._ai_activity_suffix = ""
            self.sub_title = ""
        else:
            self._ai_header_pulse = True
            if not (self._ai_activity_suffix or "").strip():
                self._ai_activity_suffix = "AI 处理中…"
            self._apply_ai_header_subtitle()
        try:
            hdr = self.query_one("#app-header", Header)
            hdr.set_class(busy, "-ai-busy")
        except Exception:
            pass

    def _drain_main_chat_queue(self) -> int:
        n = 0
        while True:
            try:
                self._chat_q.get_nowait()
                n += 1
            except asyncio.QueueEmpty:
                break
        return n

    def _interrupt_current_main_chat_turn(self, log: RichLog, tag: str) -> None:
        ct = self._current_chat_turn
        if ct is not None and not ct.done():
            ct.cancel()
            log.write(Text(f"{tag} 已请求中断当前主 AI 轮次…", style="yellow"))

    async def _main_chat_stop(
        self, log: RichLog, *, tag: str, session_event: str
    ) -> None:
        """中断当前主对话 ``agent.chat`` 协程并清空排队中的用户消息；不碰任务库与后台 task。"""
        ct0 = self._current_chat_turn
        had_active = ct0 is not None and not ct0.done()
        self._interrupt_current_main_chat_turn(log, tag)
        dropped = self._drain_main_chat_queue()
        if dropped:
            log.write(
                Text(
                    f"{tag} 已丢弃排队中的 {dropped} 条主对话请求",
                    style="cyan",
                )
            )
        if had_active or dropped:
            log.write(
                Text(
                    f"{tag} 后台 task_ai/task_shell 未停止；需要时用 /task list、task_cancel 或 /task clear。",
                    style="dim cyan",
                )
            )
        else:
            log.write(
                Text(
                    f"{tag} 当前无进行中的主 AI 轮次，队列为空。",
                    style="dim cyan",
                )
            )
        try:
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                session_event,
                "",
                f"dropped={dropped} had_active_turn={had_active}",
            )
        except Exception:
            pass

    def action_cancel_chat(self) -> None:
        t = self._current_chat_turn
        if t is not None and not t.done():
            t.cancel()
            self.query_one("#chat", RichLog).write(
                Text("[已发送中断，正在停止…]", style="yellow")
            )

    async def on_unmount(self) -> None:
        await self._shutdown_safe()

    async def _shutdown_safe(self) -> None:
        """安全退出：停聊天 Worker、取消后台任务、关库、停事件总线、关日志（幂等）。"""
        if self._shutdown_once:
            return
        self._shutdown_once = True
        try:
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "shutdown_begin",
                "",
                "",
            )
        except Exception:
            pass
        try:
            ct = self._current_chat_turn
            if ct is not None and not ct.done():
                ct.cancel()
        except Exception:
            pass
        try:
            pt = self._chat_pump_task
            if pt is not None and not pt.done():
                pt.cancel()
                try:
                    await asyncio.wait_for(pt, timeout=10.0)
                except asyncio.TimeoutError:
                    try:
                        self.ctx.session_log.write_event(
                            "tui",
                            "AicdTui",
                            "chat_pump_join_timeout",
                            "",
                            "10s",
                        )
                    except Exception:
                        pass
                except asyncio.CancelledError:
                    pass
        except Exception:
            pass
        for _ in range(15):
            await asyncio.sleep(0.04)
        try:
            await self.ctx.shutdown_graceful()
        except Exception:
            pass
        if self._heartbeat_task:
            self._heartbeat_task.cancel()
            try:
                await self._heartbeat_task
            except asyncio.CancelledError:
                pass
            self._heartbeat_task = None
        if self._bus_task:
            self._bus_task.cancel()
            try:
                await self._bus_task
            except asyncio.CancelledError:
                pass
            self._bus_task = None
        try:
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "shutdown_end",
                "",
                "ok",
            )
        except Exception:
            pass
        self.ctx.session_log.close()

    async def _bus_loop(self) -> None:
        q = self.ctx.bus.subscribe()
        try:
            while True:
                ev: dict[str, Any] = await q.get()
                await self._handle_bus_event(ev)
        except asyncio.CancelledError:
            self.ctx.bus.unsubscribe(q)
            raise

    async def _handle_bus_event(self, ev: dict[str, Any]) -> None:
        topic = ev.get("topic")
        if topic == "task":
            tid = ev.get("task_id")
            event = ev.get("event")
            if event == "history_cleared":
                self._selected_task = None
                tlog = self.query_one("#tasklog", RichLog)
                tlog.clear()
                tlog.write(Text("[任务历史已清空]", style="cyan"))
                await self.refresh_task_table()
                return
            if event == "log" and tid == self._selected_task:
                line = ev.get("line", "")
                self.query_one("#tasklog", RichLog).write(Text(line))
            if event == "status":
                await self.refresh_task_table()
                if tid == self._selected_task:
                    st = ev.get("status", "")
                    self.query_one("#tasklog", RichLog).write(
                        Text(f"[status] {tid} -> {st}", style="cyan")
                    )
            if event == "progress" and tid == self._selected_task:
                msg = ev.get("message")
                line = str(msg).strip() if msg is not None else ""
                if not line:
                    phase = ev.get("phase", "")
                    if phase == "sub_agent":
                        lr = ev.get("llm_round")
                        mr = ev.get("max_llm_rounds")
                        lt = ev.get("last_tool", "")
                        line = f"[进度] LLM {lr}/{mr}" + (
                            f" 末次工具 {lt}" if lt else ""
                        )
                    elif ev.get("percent") is not None:
                        line = f"[进度] {ev.get('percent')}% {ev.get('message', '')}"
                    else:
                        line = json.dumps(
                            {k: v for k, v in ev.items() if k not in ("topic", "event")},
                            ensure_ascii=False,
                        )[:500]
                self.query_one("#tasklog", RichLog).write(Text(line, style="cyan"))
        elif topic == "agent":
            if ev.get("event") == "phase":
                ph = str(ev.get("phase") or "")
                if ph == "llm":
                    self._ai_activity_suffix = str(ev.get("detail") or "等待大模型…")
                elif ph == "tool":
                    nm = str(ev.get("name") or "")
                    self._ai_activity_suffix = str(
                        ev.get("detail") or (f"工具 {nm}…" if nm else "执行工具…")
                    )
                else:
                    self._ai_activity_suffix = str(ev.get("detail") or "处理中…")
                if self._ai_header_busy:
                    self._apply_ai_header_subtitle()
                return
            if ev.get("event") == "tool":
                name = ev.get("name", "")
                prev = ev.get("preview", "")
                self.query_one("#chat", RichLog).write(
                    Text(f"[tool {name}] {prev}", style="dim")
                )
            elif ev.get("event") == "log":
                self.query_one("#chat", RichLog).write(Text(ev.get("line", "")))

    async def refresh_task_table(self) -> None:
        table = self.query_one("#tasks", DataTable)
        table.clear(columns=True)
        table.add_columns("id", "type", "status")
        rows = await self.ctx.tasks.list_tasks(80)
        active_states = frozenset({"running", "pending"})

        def _ts(r: Any) -> str:
            return getattr(r, "updated_at", None) or ""

        running = [r for r in rows if (getattr(r, "status", "") or "").lower() in active_states]
        done = [r for r in rows if (getattr(r, "status", "") or "").lower() not in active_states]
        running.sort(key=_ts, reverse=True)
        done.sort(key=_ts, reverse=True)
        # 运行中/排队置顶，便于发现新起的子任务；已完成仍保留（SQLite 持久化，除非 /task clear）
        for r in running + done:
            short_id = r.id[:8] + "…" if len(r.id) > 10 else r.id
            table.add_row(short_id, r.type, r.status, key=r.id)

    async def on_data_table_row_highlighted(
        self, event: DataTable.RowHighlighted
    ) -> None:
        await self._show_task_detail(event.row_key)

    async def on_data_table_row_selected(self, event: DataTable.RowSelected) -> None:
        await self._show_task_detail(event.row_key)

    async def _show_task_detail(self, row_key: Any) -> None:
        val = getattr(row_key, "value", None)
        if val is None:
            return
        tid = cast(str, val)
        self._selected_task = tid
        log = self.query_one("#tasklog", RichLog)
        log.clear()
        log.write(Text(f"选中任务: {tid}", style="bold"))
        row = await self.ctx.tasks.get_task_row(tid)
        if row:
            log.write(Text(f"type={row.type} status={row.status}", style="dim"))
            log.write(
                Text(
                    f"queued={row.created_at}  updated={row.updated_at}",
                    style="dim",
                )
            )
            pl = row.payload or {}
            dt = pl.get("display_title")
            if isinstance(dt, str) and dt.strip():
                log.write(Text(f"标题: {dt.strip()[:500]}", style="bold"))
            pu = pl.get("purpose")
            if isinstance(pu, str) and pu.strip():
                log.write(Text(f"目的: {pu.strip()[:1200]}", style="dim"))
            for key, label in (
                ("started_at", "开始(UTC)"),
                ("finished_at", "结束(UTC)"),
                ("duration_sec", "耗时秒"),
            ):
                v = pl.get(key)
                if v is not None and str(v).strip():
                    log.write(Text(f"{label}: {v}", style="dim"))
            if row.status == "running":
                sa = pl.get("started_at")
                if isinstance(sa, str) and sa.strip():
                    try:
                        t0 = datetime.fromisoformat(sa.replace("Z", "+00:00"))
                        if t0.tzinfo is None:
                            t0 = t0.replace(tzinfo=timezone.utc)
                        elapsed = int(
                            (datetime.now(timezone.utc) - t0).total_seconds()
                        )
                        log.write(
                            Text(f"已运行约 {elapsed}s（进行中）", style="yellow")
                        )
                    except ValueError:
                        pass
            est = pl.get("estimated_steps")
            if est is not None:
                log.write(Text(f"预计步骤数: {est}", style="dim"))
            if row.type == "shell":
                cmd = pl.get("command")
                if isinstance(cmd, str) and cmd.strip():
                    log.write(Text(f"命令: {cmd.strip()[:2000]}", style="dim"))
                sp = pl.get("shell_process")
                if isinstance(sp, dict) and sp:
                    log.write(
                        Text(
                            "Shell 进程: "
                            + json.dumps(sp, ensure_ascii=False)[:1200],
                            style="cyan",
                        )
                    )
            prog = pl.get("progress")
            if isinstance(prog, dict) and prog:
                log.write(
                    Text(
                        "进度: "
                        + json.dumps(prog, ensure_ascii=False)[:1600],
                        style="cyan",
                    )
                )
            trace = pl.get("sub_agent_trace")
            if isinstance(trace, dict) and trace:
                log.write(
                    Text(
                        "子Agent轨迹: "
                        + json.dumps(trace, ensure_ascii=False)[:1600],
                        style="cyan",
                    )
                )
            if row.result_summary:
                log.write(Text(str(row.result_summary)[:2000]))
            if row.error:
                log.write(Text(str(row.error), style="red"))

    def _input_history_path(self) -> Path:
        return Path(self.ctx.layout["data"]) / self._input_history_filename

    def _load_input_history_from_disk(self) -> None:
        path = self._input_history_path()
        if not path.is_file():
            return
        cap = self._input_history_cap
        loaded: list[str] = []
        try:
            with path.open("r", encoding="utf-8") as f:
                for raw in f:
                    line = raw.strip()
                    if not line:
                        continue
                    try:
                        loaded.append(json.loads(line))
                    except json.JSONDecodeError:
                        continue
        except OSError:
            return
        if len(loaded) > cap:
            loaded = loaded[-cap:]
        self._input_history = loaded

    def _persist_input_history(self) -> None:
        path = self._input_history_path()
        path.parent.mkdir(parents=True, exist_ok=True)
        tmp = path.with_suffix(".jsonl.tmp")
        try:
            with tmp.open("w", encoding="utf-8", newline="\n") as f:
                for item in self._input_history:
                    f.write(json.dumps(item, ensure_ascii=False) + "\n")
            tmp.replace(path)
        except OSError:
            try:
                if tmp.is_file():
                    tmp.unlink(missing_ok=True)
            except OSError:
                pass

    def _push_input_history(self, text: str) -> None:
        cap = self._input_history_cap
        self._input_history.append(text)
        if len(self._input_history) > cap:
            del self._input_history[: len(self._input_history) - cap]
        self._persist_input_history()

    def _input_history_prompt_only(self) -> list[str]:
        """仅发给主 AI 的提示词（不以 / 开头的输入）；/ 命令仍写入磁盘但不参与 list/N 编号。"""
        return [
            x
            for x in self._input_history
            if x and not x.lstrip().startswith("/")
        ]

    def _copy_plain_for_user(self, text: str) -> str:
        """同步到系统剪贴板并返回一行状态说明（RichLog 在备用屏中无法用鼠标拖选）。"""
        try:
            self.copy_to_clipboard(text)
        except Exception:
            pass
        if _try_write_system_clipboard(text):
            return "已写入系统剪贴板，可在其他窗口 Ctrl+V（macOS：Cmd+V）粘贴。"
        return (
            "未能调用系统剪贴板；已尝试终端 OSC52。"
            "可换 Windows Terminal，或使用 /history copy <N> 仅再试剪贴板。"
        )

    @staticmethod
    def _history_slash_help() -> str:
        return (
            "[/history] 底部输入历史：磁盘仍保存全部行；"
            "**list / N 仅针对发给主 AI 的提示词**（不以 / 开头的输入），本地 / 命令不列入编号。\n"
            "说明：左侧为 Textual 绘制区，底部为多行输入（Enter 发送、Shift+Enter 换行）；**一般不能像普通终端那样用鼠标拖选复制**；"
            "下面命令会尽量把全文写入**系统剪贴板**。\n"
            "最近条目持久化在 HOME/data/tui_input_history.jsonl，跨重启保留。\n"
            "  /history list          提示词编号清单（1=最旧，数字越大越新）\n"
            "  /history clear         清空内存与上述文件中的**全部**记录（含 / 命令行）\n"
            "  /history <N>           打印第 N 条提示词全文，并尝试写入系统剪贴板\n"
            "  /history copy <N>      仅写入剪贴板（不刷屏；N 与 list 一致）\n"
            "  /history <N> copy      同上\n"
            "  /history help          本说明"
        )

    @on(ChatTextArea.Submitted)
    async def on_chat_text_area_submitted(self, event: ChatTextArea.Submitted) -> None:
        if event.chat.id != "msg":
            return
        text = event.value.strip()
        if not text:
            return
        self._push_input_history(text)
        self.query_one("#chat", RichLog).write(Text(f"[你] {text}", style="green"))
        parsed = parse_slash_line(text)
        if parsed is not None:
            await self._run_slash_command(parsed[0], parsed[1])
            return
        await self._chat_q.put(text)
        turn_busy = (
            self._current_chat_turn is not None
            and not self._current_chat_turn.done()
        )
        if turn_busy:
            nq = self._chat_q.qsize()
            suffix = f"（队列中约 {nq} 条）" if nq > 0 else ""
            self.query_one("#chat", RichLog).write(
                Text(
                    f"[系统] 主 AI 仍处理上一条；本条已排队，将依次执行{suffix}",
                    style="dim cyan",
                )
            )

    async def _idle_heartbeat_loop(self) -> None:
        """主对话空闲时按配置注入定时检查（与真人消息共用队列，有输入时让路）。"""
        try:
            while not self._shutdown_once:
                sec = int(self.ctx.config.agent.idle_heartbeat_sec or 0)
                if sec <= 0:
                    await asyncio.sleep(12)
                    continue
                wait = max(5, min(3600, sec))
                await asyncio.sleep(wait)
                if self._shutdown_once:
                    break
                ct = self._current_chat_turn
                if ct is not None and not ct.done():
                    continue
                if self._chat_q.qsize() > 0:
                    continue
                from aicd.agent.idle_pulse import main_idle_pulse_user_text

                await self._chat_q.put(main_idle_pulse_user_text())
        except asyncio.CancelledError:
            raise

    async def _chat_pump_loop(self) -> None:
        """顺序处理用户消息：新输入排队，不再取消上一轮（避免误显「助手回复已中断」）。"""
        try:
            while True:
                text = await self._chat_q.get()
                if is_main_idle_pulse(text):
                    self.query_one("#chat", RichLog).write(
                        Text(
                            "[系统] 主对话定时心跳：检查后台子任务…",
                            style="dim cyan",
                        )
                    )
                self._set_ai_header_busy(True)
                turn = asyncio.create_task(self.ctx.agent.chat(text))
                self._current_chat_turn = turn
                try:
                    reply = await turn
                except asyncio.CancelledError:
                    # /exit 会先置 _shutdown_once 再 cancel turn；勿误报为「中断」
                    if not self._shutdown_once:
                        self.query_one("#chat", RichLog).write(
                            Text("[助手回复已中断]", style="yellow")
                        )
                except Exception as e:  # noqa: BLE001
                    self.ctx.session_log.write_exception(
                        "tui",
                        "AicdTui",
                        "chat_turn",
                        e,
                    )
                    err = f"错误: {e}"
                    self.query_one("#chat", RichLog).write(
                        Text(f"[助手] {err}", style="bold")
                    )
                else:
                    self.query_one("#chat", RichLog).write(
                        Text(f"[助手] {reply}", style="bold")
                    )
                finally:
                    self._current_chat_turn = None
                    self._set_ai_header_busy(False)
        except asyncio.CancelledError:
            raise

    async def _run_slash_command(self, cmd: str, args: list[str]) -> None:
        log = self.query_one("#chat", RichLog)
        if cmd == "help":
            log.write(Text(slash_help_text(), style="cyan"))
            return
        if cmd == "chat":
            await self._slash_chat(args)
            return
        if cmd == "history":
            if not args or (
                len(args) == 1 and str(args[0]).strip().lower() in ("help", "h", "?")
            ):
                log.write(Text(self._history_slash_help(), style="cyan"))
                return
            if len(args) >= 2:
                a0 = str(args[0]).strip().lower()
                a1 = str(args[1]).strip()
                n_copy: int | None = None
                if a0 == "copy" and a1.isdigit():
                    n_copy = int(a1)
                elif str(args[0]).strip().isdigit() and a1.lower() == "copy":
                    n_copy = int(str(args[0]).strip())
                if n_copy is not None:
                    prompts = self._input_history_prompt_only()
                    if n_copy < 1 or n_copy > len(prompts):
                        mx = len(prompts)
                        log.write(
                            Text(
                                f"[/history] 无效序号 {n_copy}（当前提示词共 {mx} 条，有效 1–{mx}）"
                                if mx
                                else "[/history] 暂无提示词记录（以 / 开头的本地命令不列入编号）",
                                style="yellow",
                            )
                        )
                        return
                    chosen = prompts[n_copy - 1]
                    clip_msg = self._copy_plain_for_user(chosen)
                    log.write(
                        Text(
                            f"[/history] 第 {n_copy} 条提示词（约 {len(chosen)} 字符）。{clip_msg}",
                            style="cyan",
                        )
                    )
                    return
            sub = str(args[0]).strip()
            low = sub.lower()
            if low == "list":
                prompts = self._input_history_prompt_only()
                if not prompts:
                    log.write(
                        Text(
                            "[/history] 暂无提示词记录（以 / 开头的本地命令不列入清单）",
                            style="yellow",
                        )
                    )
                    return
                lines: list[str] = [
                    "[/history] 提示词清单（与 /history N 的 N 一致；不含 / 命令）：",
                    "",
                ]
                for i, line in enumerate(prompts, start=1):
                    preview = line if len(line) <= 140 else line[:137] + "…"
                    lines.append(f"  {i}.  {preview}")
                log.write(Text("\n".join(lines), style="cyan"))
                return
            if low == "clear":
                n = len(self._input_history)
                self._input_history.clear()
                self._persist_input_history()
                log.write(
                    Text(
                        f"[/history] 已清空输入历史（共删除 {n} 条，含 / 命令行）",
                        style="cyan",
                    )
                )
                return
            if sub.isdigit():
                n = int(sub)
                prompts = self._input_history_prompt_only()
                if n < 1 or n > len(prompts):
                    mx = len(prompts)
                    log.write(
                        Text(
                            f"[/history] 无效序号 {n}（当前提示词共 {mx} 条，有效 1–{mx}）"
                            if mx
                            else "[/history] 暂无提示词记录（以 / 开头的本地命令不列入编号）",
                            style="yellow",
                        )
                    )
                    return
                chosen = prompts[n - 1]
                clip_msg = self._copy_plain_for_user(chosen)
                log.write(
                    Text(
                        f"[/history] 第 {n} 条提示词（全文如下）。{clip_msg}",
                        style="cyan",
                    )
                )
                log.write(Text(chosen, style="white"))
                log.write(Text("— 全文结束 —", style="dim cyan"))
                return
            log.write(
                Text(
                    f"[/history] 未知参数 {sub!r}，使用 /history list | clear | copy <N> | help",
                    style="yellow",
                )
            )
            return
        if cmd == "work":
            path_arg = " ".join(args).strip()
            ok, msg = self.ctx.set_ai_workspace(path_arg or None)
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "/work",
                path_arg or "(query)",
                f"ok={ok} {msg[:2000]}",
            )
            if ok:
                log.write(Text(f"[/work] AI 工作目录: {msg}", style="cyan"))
            else:
                log.write(Text(f"[/work] 失败: {msg}", style="red"))
            return
        if cmd == "doc":
            path_arg = " ".join(args).strip()
            ok, msg = self.ctx.set_ai_output_dir(path_arg or None)
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "/doc",
                path_arg or "(query)",
                f"ok={ok} {msg[:2000]}",
            )
            if ok:
                log.write(Text(f"[/doc] {msg}", style="cyan"))
            else:
                log.write(Text(f"[/doc] 失败: {msg}", style="red"))
            return
        if cmd == "llm":
            ok, msg = self.ctx.apply_slash_llm(args)
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "/llm",
                repr(args)[:500],
                f"ok={ok}",
            )
            log.write(
                Text(
                    msg if ok else f"失败: {msg}",
                    style="cyan" if ok else "red",
                )
            )
            return
        if cmd == "agent":
            ok, msg = self.ctx.apply_slash_agent(args)
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "/agent",
                repr(args)[:500],
                f"ok={ok}",
            )
            log.write(
                Text(
                    msg if ok else f"失败: {msg}",
                    style="cyan" if ok else "red",
                )
            )
            return
        if cmd == "task":
            await self._slash_task(args)
            return
        if cmd in ("exit", "quit", "q"):
            log.write(
                Text(
                    "[/exit] 正在安全退出：停止聊天、取消后台任务、关闭数据库…",
                    style="cyan",
                )
            )
            try:
                self.ctx.session_log.write_event(
                    "tui",
                    "AicdTui",
                    f"/{cmd}",
                    "",
                    "safe_exit_begin",
                )
            except Exception:
                pass
            await self._shutdown_safe()
            try:
                self.query_one("#chat", RichLog).write(
                    Text("[/exit] 已停止后台任务并关闭数据库，再见。", style="dim cyan")
                )
            except Exception:
                pass
            self.exit()
            return
        log.write(Text(f"未知命令 /{cmd}，输入 /? 查看列表", style="yellow"))

    async def _slash_chat(self, args: list[str]) -> None:
        log = self.query_one("#chat", RichLog)
        if not args or str(args[0]).strip().lower() in ("help", "?", "h"):
            log.write(Text(chat_slash_help_text(), style="cyan"))
            return
        sub = str(args[0]).strip().lower()
        if sub == "stop":
            await self._main_chat_stop(
                log, tag="[/chat stop]", session_event="/chat stop"
            )
            return
        log.write(
            Text(
                f"未知 /chat 子命令 {sub!r}，使用 /chat help（仅支持 stop）",
                style="yellow",
            )
        )

    async def _slash_task(self, args: list[str]) -> None:
        log = self.query_one("#chat", RichLog)
        if not args or args[0].lower() in ("help", "?", "h"):
            log.write(Text(task_slash_help_text(), style="cyan"))
            return
        sub = args[0].lower()
        if sub == "stop":
            await self._main_chat_stop(
                log, tag="[/task stop]", session_event="/task stop"
            )
            return
        if sub == "list":
            await self.refresh_task_table()
            rows = await self.ctx.tasks.list_tasks(80)
            log.write(
                Text(
                    f"[/task list] 已刷新任务表（约 {len(rows)} 条，按更新时间排序）。",
                    style="cyan",
                )
            )
            return
        if sub == "clear":
            # 先停主对话当前轮并丢弃排队请求，避免清空后台后模型仍按旧指令继续 task_ai
            self._interrupt_current_main_chat_turn(log, "[/task clear]")
            dropped = self._drain_main_chat_queue()
            if dropped:
                log.write(
                    Text(
                        f"[/task clear] 已丢弃排队中的 {dropped} 条主对话请求",
                        style="cyan",
                    )
                )
            try:
                result = await self.ctx.tasks.clear_task_history()
            except Exception as e:  # noqa: BLE001
                self.ctx.session_log.write_exception(
                    "tui",
                    "AicdTui",
                    "/task clear",
                    e,
                )
                log.write(Text(f"[/task clear] 失败: {e}", style="red"))
                return
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "/task clear",
                "",
                str(result)[:2000],
            )
            self._selected_task = None
            tlog = self.query_one("#tasklog", RichLog)
            tlog.clear()
            tlog.write(Text("[任务历史已清空]", style="cyan"))
            await self.refresh_task_table()
            log.write(
                Text(
                    "[/task clear] 已清空：取消运行中任务，删除库中任务与 task_messages，"
                    f"清理工作区目录（详情: {result}）",
                    style="cyan",
                )
            )
            return
        if sub == "ai":
            instruction = " ".join(args[1:]).strip()
            if not instruction:
                log.write(
                    Text(
                        "用法: /task ai <说明…>  例: /task ai 在工作区统计所有 .py 行数",
                        style="yellow",
                    )
                )
                return
            tid = await self.ctx.tasks.create_ai_task(instruction)
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "/task ai",
                instruction[:2000],
                f"task_id={tid}",
            )
            log.write(
                Text(
                    f"[/task ai] 已排队后台子 Agent\n"
                    f"  task_id={tid}\n"
                    f"右侧任务表与日志可跟踪；也可让主对话用 task_result 查询。",
                    style="cyan",
                )
            )
            await self.refresh_task_table()
            return
        if sub == "shell":
            cmd = join_args_for_shell(args[1:])
            if not cmd.strip():
                log.write(
                    Text(
                        '用法: /task shell <命令>  例: /task shell curl -L -o tmp/dl.zip URL',
                        style="yellow",
                    )
                )
                return
            try:
                tid = await self.ctx.tasks.create_shell_task(cmd)
            except ValueError as e:
                log.write(Text(str(e), style="red"))
                return
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "/task shell",
                cmd[:2000],
                f"task_id={tid}",
            )
            log.write(
                Text(
                    f"[/task shell] 已排队后台 Shell\n"
                    f"  task_id={tid}\n"
                    f"输出在右侧任务日志；主对话可用 task_list / task_result 查看状态。",
                    style="cyan",
                )
            )
            await self.refresh_task_table()
            return
        if sub == "dummy":
            steps = 5
            if len(args) > 1:
                try:
                    steps = max(1, min(500, int(args[1])))
                except ValueError:
                    log.write(Text("dummy 步数须为整数", style="yellow"))
                    return
            tid = await self.ctx.tasks.create_dummy_long_task(steps=steps)
            self.ctx.session_log.write_event(
                "tui",
                "AicdTui",
                "/task dummy",
                f"steps={steps}",
                f"task_id={tid}",
            )
            log.write(
                Text(f"[/task dummy] task_id={tid} steps={steps}", style="cyan")
            )
            await self.refresh_task_table()
            return
        log.write(
            Text(
                f"未知子命令 {sub!r}，输入 /task help",
                style="yellow",
            )
        )




"""会话 id：每次 new_chat_session_id 覆盖文件且 id 不同。"""

from __future__ import annotations

from pathlib import Path

from aicd.home import new_chat_session_id


def test_new_chat_session_id_always_fresh(tmp_path: Path) -> None:
    data = tmp_path / "data"
    a = new_chat_session_id(data)
    b = new_chat_session_id(data)
    assert a != b
    path = data / "active_chat_session_id.txt"
    assert path.read_text(encoding="utf-8").strip() == b



from __future__ import annotations

import os
import uuid
from pathlib import Path

from aicd.config import AppConfig, load_config, save_config


def default_home() -> Path:
    """未设置 AICD_HOME 且未传 --home 时，使用当前用户主目录下的 ``~/.aicd``。"""
    env = os.environ.get("AICD_HOME")
    if env:
        return Path(env).expanduser().resolve()
    return (Path.home() / ".aicd").resolve()


def home_paths(root: Path) -> dict[str, Path]:
    return {
        "root": root,
        "config": root / "config.yaml",
        "data": root / "data",
        "db": root / "data" / "app.db",
        "tasks": root / "tasks",
        "cache": root / "cache",
        "logs": root / "logs",
        "tmp_scripts": root / "tmp" / "scripts",
    }


def new_chat_session_id(data_dir: Path) -> str:
    """生成新的主会话 UUID 并写入 ``data/active_chat_session_id.txt``（每次调用均换新 id）。"""
    data_dir.mkdir(parents=True, exist_ok=True)
    path = data_dir / "active_chat_session_id.txt"
    sid = str(uuid.uuid4())
    path.write_text(sid, encoding="utf-8")
    return sid


def ensure_home_layout(root: Path) -> dict[str, Path]:
    paths = home_paths(root)
    root.mkdir(parents=True, exist_ok=True)
    for key in ("data", "tasks", "cache", "logs", "tmp_scripts"):
        paths[key].mkdir(parents=True, exist_ok=True)
    return paths


def init_home(root: Path | None = None) -> tuple[Path, dict[str, Path], AppConfig]:
    r = root or default_home()
    layout = ensure_home_layout(r)
    cfg_path = layout["config"]
    if not cfg_path.is_file():
        save_config(cfg_path, AppConfig())
    cfg = load_config(cfg_path)
    return r, layout, cfg



"""MySQL / PostgreSQL / SQLite 异步访问（供 Agent 在用户提供连接信息时使用）。

PostgreSQL 上带 RETURNING 的 INSERT 等会走 execute 路径，不返回结果集；需要时用只读 SELECT 再查。
"""

from __future__ import annotations

import asyncio
import re
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Sequence

WRITE_CONFIRM = "EXECUTE_WRITE"
_MAX_SQL_CHARS = 200_000
_DEFAULT_TIMEOUT = 60.0


@dataclass(frozen=True)
class DbConnectionSpec:
    engine: str  # sqlite | postgresql | mysql
    path: str | None = None
    host: str | None = None
    port: int | None = None
    user: str | None = None
    password: str | None = None
    database: str | None = None
    schema: str | None = None  # PG default schema / search_path hint
    sslmode: str | None = None


def connection_from_dict(d: dict[str, Any]) -> DbConnectionSpec:
    eng = str(d.get("engine") or "").strip().lower()
    if eng in ("pg", "postgres"):
        eng = "postgresql"
    if eng not in ("sqlite", "postgresql", "mysql"):
        raise ValueError(f"unsupported engine: {eng!r} (use sqlite|postgresql|mysql)")
    path = d.get("path")
    path_s = str(path).strip() if path else None
    port = d.get("port")
    pi = int(port) if port is not None else None
    return DbConnectionSpec(
        engine=eng,
        path=path_s,
        host=str(d["host"]).strip() if d.get("host") else None,
        port=pi,
        user=str(d["user"]).strip() if d.get("user") else None,
        password=str(d["password"]) if d.get("password") is not None else None,
        database=str(d["database"]).strip() if d.get("database") else None,
        schema=str(d["schema"]).strip() if d.get("schema") else None,
        sslmode=str(d["sslmode"]).strip() if d.get("sslmode") else None,
    )


def validate_connection(spec: DbConnectionSpec) -> None:
    if spec.engine == "sqlite":
        if not spec.path:
            raise ValueError("sqlite requires connection.path")
        return
    if not spec.host:
        raise ValueError(f"{spec.engine} requires connection.host")
    if not spec.user:
        raise ValueError(f"{spec.engine} requires connection.user")
    if spec.password is None:
        raise ValueError(f"{spec.engine} requires connection.password (may be empty string)")
    if not spec.database:
        raise ValueError(f"{spec.engine} requires connection.database")


def _safe_ident(name: str) -> str:
    s = name.strip()
    if not re.match(r"^[a-zA-Z_][a-zA-Z0-9_]*$", s):
        raise ValueError(f"invalid SQL identifier: {name!r}")
    return s


def _strip_leading_sql_comments(sql: str) -> str:
    s = sql.strip()
    while True:
        if s.startswith("--"):
            nl = s.find("\n")
            if nl == -1:
                return ""
            s = s[nl + 1 :].lstrip()
            continue
        if s.startswith("/*"):
            end = s.find("*/", 2)
            if end == -1:
                return s
            s = s[end + 2 :].lstrip()
            continue
        break
    return s


def first_statement_only(sql: str) -> str:
    """只取第一条语句（分号分隔），降低多语句注入面。"""
    body = sql.strip()
    depth = 0
    in_single = False
    in_double = False
    i = 0
    while i < len(body):
        c = body[i]
        if in_single:
            if c == "'" and body[i - 1 : i + 2] != "''":
                in_single = False
            i += 1
            continue
        if in_double:
            if c == '"':
                in_double = False
            i += 1
            continue
        if c == "'":
            in_single = True
            i += 1
            continue
        if c == '"':
            in_double = True
            i += 1
            continue
        if c == "(":
            depth += 1
        elif c == ")" and depth > 0:
            depth -= 1
        elif c == ";" and depth == 0:
            return body[:i].strip()
        i += 1
    return body.strip()


def is_readonly_sql(sql: str) -> bool:
    """粗略判断只读：首条语句以 SELECT/WITH/SHOW/DESC/DESCRIBE/EXPLAIN 开头。"""
    one = first_statement_only(sql)
    head = _strip_leading_sql_comments(one)
    if not head:
        return False
    return bool(
        re.match(
            r"(select|with|show|describe|desc|explain)\b",
            head,
            re.IGNORECASE | re.DOTALL,
        )
    )


def _json_cell(v: Any) -> Any:
    if v is None:
        return None
    if isinstance(v, (bool, int, float, str)):
        return v
    if isinstance(v, bytes):
        hx = v[:64].hex()
        return f"<bytes len={len(v)} hex64={hx}>"
    return str(v)


def _rows_to_json(
    columns: list[str], rows: Sequence[Sequence[Any]], max_rows: int
) -> tuple[list[dict[str, Any]], bool]:
    out: list[dict[str, Any]] = []
    truncated = False
    for i, row in enumerate(rows):
        if i >= max_rows:
            truncated = True
            break
        out.append({columns[j]: _json_cell(row[j]) for j in range(len(columns))})
    return out, truncated


async def _run_sqlite(
    path: Path,
    sql: str,
    *,
    read_only: bool,
    max_rows: int,
) -> dict[str, Any]:
    import aiosqlite

    mode = "ro" if read_only else "rwc"
    uri = f"file:{path.as_posix()}?mode={mode}"
    async with aiosqlite.connect(uri, uri=True) as db:
        db.row_factory = aiosqlite.Row
        try:
            async with db.execute(sql) as cur:
                if cur.description:
                    cols = [d[0] for d in cur.description]
                    raw = await cur.fetchmany(max_rows + 1)
                    rows, trunc = _rows_to_json(cols, raw, max_rows)
                    return {
                        "ok": True,
                        "columns": cols,
                        "rows": rows,
                        "truncated": trunc,
                        "rowcount": cur.rowcount if cur.rowcount >= 0 else None,
                    }
                rc = cur.rowcount
            if not read_only:
                await db.commit()
            return {
                "ok": True,
                "columns": [],
                "rows": [],
                "truncated": False,
                "rowcount": rc,
            }
        except Exception as e:  # noqa: BLE001
            await db.rollback()
            return {"ok": False, "error": repr(e)}


def _pg_stmt_returns_rows(stmt: str) -> bool:
    s = _strip_leading_sql_comments(stmt).lstrip().lower()
    if re.match(
        r"(select|with|show|values|table|explain|describe)\b",
        s,
    ):
        return True
    return s.startswith("desc ") or s.startswith("desc\t")


def _pg_stmt_is_mutating(stmt: str) -> bool:
    s = _strip_leading_sql_comments(stmt).lstrip().lower()
    return bool(
        re.match(
            r"(insert|update|delete|create|drop|alter|truncate|grant|revoke)\b",
            s,
        )
    )


async def _run_postgresql(
    spec: DbConnectionSpec,
    sql: str,
    *,
    read_only: bool,
    max_rows: int,
) -> dict[str, Any]:
    try:
        import asyncpg
    except ImportError as e:
        return {
            "ok": False,
            "error": f"asyncpg not installed: {e}. pip install 'aicd[db]'",
        }
    port = spec.port or 5432
    kwargs: dict[str, Any] = {
        "host": spec.host,
        "port": port,
        "user": spec.user,
        "password": spec.password,
        "database": spec.database,
        "timeout": _DEFAULT_TIMEOUT,
    }
    if spec.sslmode:
        kwargs["ssl"] = spec.sslmode != "disable"
    conn = await asyncpg.connect(**kwargs)
    try:
        if spec.schema:
            sch = _safe_ident(spec.schema)
            await conn.execute(f"SET search_path TO {sch}")
        stmt = first_statement_only(sql)
        use_fetch = read_only or (
            not _pg_stmt_is_mutating(stmt) and _pg_stmt_returns_rows(stmt)
        )
        if use_fetch:
            rows = await conn.fetch(stmt)
            if not rows:
                return {
                    "ok": True,
                    "columns": [],
                    "rows": [],
                    "truncated": False,
                    "rowcount": 0,
                }
            cols = list(rows[0].keys())
            data = [tuple(r[c] for c in cols) for r in rows[: max_rows + 1]]
            out, trunc = _rows_to_json(cols, data, max_rows)
            return {
                "ok": True,
                "columns": cols,
                "rows": out,
                "truncated": trunc,
                "rowcount": len(rows),
            }
        status = await conn.execute(stmt)
        return {
            "ok": True,
            "columns": [],
            "rows": [],
            "truncated": False,
            "status": str(status),
            "rowcount": None,
        }
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": repr(e)}
    finally:
        await conn.close()


async def _run_mysql(
    spec: DbConnectionSpec,
    sql: str,
    *,
    read_only: bool,
    max_rows: int,
) -> dict[str, Any]:
    try:
        import aiomysql
    except ImportError as e:
        return {
            "ok": False,
            "error": f"aiomysql not installed: {e}. pip install 'aicd[db]'",
        }
    port = spec.port or 3306
    conn = await aiomysql.connect(
        host=spec.host,
        port=port,
        user=spec.user,
        password=spec.password,
        db=spec.database,
        autocommit=True,
    )
    try:
        async with conn.cursor(aiomysql.DictCursor) as cur:
            await cur.execute(sql)
            if cur.description:
                cols = [d[0] for d in cur.description]
                raw = await cur.fetchmany(max_rows + 1)
                tuples = [tuple(row[c] for c in cols) for row in raw]
                out, trunc = _rows_to_json(cols, tuples, max_rows)
                return {
                    "ok": True,
                    "columns": cols,
                    "rows": out,
                    "truncated": trunc,
                    "rowcount": cur.rowcount,
                }
            return {
                "ok": True,
                "columns": [],
                "rows": [],
                "truncated": False,
                "rowcount": cur.rowcount,
            }
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": repr(e)}
    finally:
        conn.close()


async def db_execute(
    spec: DbConnectionSpec,
    sql: str,
    *,
    read_only: bool,
    max_rows: int,
    write_confirm: str | None,
) -> dict[str, Any]:
    if len(sql) > _MAX_SQL_CHARS:
        return {"ok": False, "error": f"sql exceeds {_MAX_SQL_CHARS} chars"}
    if not read_only:
        if write_confirm != WRITE_CONFIRM:
            return {
                "ok": False,
                "error": f"writes require write_confirm={WRITE_CONFIRM!r}",
            }
    else:
        if not is_readonly_sql(sql):
            return {
                "ok": False,
                "error": "read_only mode: only SELECT/WITH/SHOW/DESCRIBE/DESC/EXPLAIN allowed",
            }

    validate_connection(spec)
    full = sql.strip()
    stmt = first_statement_only(sql)
    if full != stmt:
        suffix = full[len(stmt) :].lstrip()
        if suffix.startswith(";") and suffix[1:].strip():
            return {"ok": False, "error": "multiple statements are not allowed"}

    async def _go() -> dict[str, Any]:
        if spec.engine == "sqlite":
            p = Path(spec.path or "")
            return await _run_sqlite(p, stmt, read_only=read_only, max_rows=max_rows)
        if spec.engine == "postgresql":
            return await _run_postgresql(
                spec, stmt, read_only=read_only, max_rows=max_rows
            )
        return await _run_mysql(spec, stmt, read_only=read_only, max_rows=max_rows)

    return await asyncio.wait_for(_go(), timeout=_DEFAULT_TIMEOUT + 5.0)


async def db_ping(spec: DbConnectionSpec) -> dict[str, Any]:
    validate_connection(spec)
    if spec.engine == "sqlite":
        import aiosqlite

        p = Path(spec.path or "")
        uri = f"file:{p.as_posix()}?mode=ro"
        try:
            async with aiosqlite.connect(uri, uri=True) as db:
                async with db.execute("SELECT 1") as cur:
                    await cur.fetchone()
            return {"ok": True, "engine": "sqlite", "path": str(p.resolve())}
        except Exception as e:  # noqa: BLE001
            return {"ok": False, "error": repr(e)}
    if spec.engine == "postgresql":
        r = await _run_postgresql(
            spec, "SELECT version()", read_only=True, max_rows=1
        )
        if r.get("ok") and r.get("rows"):
            return {"ok": True, "engine": "postgresql", "version": r["rows"][0]}
        return r
    r = await _run_mysql(spec, "SELECT VERSION() AS v", read_only=True, max_rows=1)
    if r.get("ok") and r.get("rows"):
        return {"ok": True, "engine": "mysql", "version": r["rows"][0]}
    return r


async def db_list_tables(spec: DbConnectionSpec) -> dict[str, Any]:
    validate_connection(spec)
    if spec.engine == "sqlite":
        sql = (
            "SELECT name FROM sqlite_master "
            "WHERE type='table' AND name NOT LIKE 'sqlite_%' ORDER BY name"
        )
        p = Path(spec.path or "")
        return await _run_sqlite(p, sql, read_only=True, max_rows=10_000)
    if spec.engine == "postgresql":
        if spec.schema:
            sch = _safe_ident(spec.schema)
            sql = f"""
                SELECT table_schema, table_name
                FROM information_schema.tables
                WHERE table_type = 'BASE TABLE'
                  AND table_schema NOT IN ('pg_catalog', 'information_schema')
                  AND table_schema = '{sch}'
                ORDER BY table_schema, table_name
            """
        else:
            sql = """
                SELECT table_schema, table_name
                FROM information_schema.tables
                WHERE table_type = 'BASE TABLE'
                  AND table_schema NOT IN ('pg_catalog', 'information_schema')
                ORDER BY table_schema, table_name
            """
        return await _run_postgresql(spec, sql, read_only=True, max_rows=10_000)
    sql = (
        "SELECT TABLE_SCHEMA AS table_schema, TABLE_NAME AS table_name "
        "FROM information_schema.tables "
        "WHERE table_type = 'BASE TABLE' AND TABLE_SCHEMA = DATABASE() "
        "ORDER BY TABLE_NAME"
    )
    return await _run_mysql(spec, sql, read_only=True, max_rows=10_000)


async def db_list_schemas(spec: DbConnectionSpec) -> dict[str, Any]:
    validate_connection(spec)
    if spec.engine == "sqlite":
        return {"ok": True, "schemas": [{"name": "main"}]}
    if spec.engine == "postgresql":
        sql = (
            "SELECT schema_name FROM information_schema.schemata "
            "WHERE schema_name NOT LIKE 'pg_%' AND schema_name != 'information_schema' "
            "ORDER BY schema_name"
        )
        return await _run_postgresql(spec, sql, read_only=True, max_rows=2000)
    sql = "SHOW DATABASES"
    return await _run_mysql(spec, sql, read_only=True, max_rows=2000)


async def db_describe_table(
    spec: DbConnectionSpec, table: str, schema: str | None = None
) -> dict[str, Any]:
    t = _safe_ident(table)
    validate_connection(spec)
    if spec.engine == "sqlite":
        sql = f"PRAGMA table_info({t})"
        p = Path(spec.path or "")
        return await _run_sqlite(p, sql, read_only=True, max_rows=500)
    if spec.engine == "postgresql":
        sch = _safe_ident(schema or spec.schema or "public")
        sql = f"""
            SELECT column_name, data_type, is_nullable, column_default
            FROM information_schema.columns
            WHERE table_schema = '{sch}' AND table_name = '{t}'
            ORDER BY ordinal_position
        """
        return await _run_postgresql(spec, sql, read_only=True, max_rows=500)
    sql = f"""
        SELECT COLUMN_NAME, DATA_TYPE, IS_NULLABLE, COLUMN_DEFAULT
        FROM information_schema.COLUMNS
        WHERE TABLE_SCHEMA = DATABASE() AND TABLE_NAME = '{t}'
        ORDER BY ORDINAL_POSITION
    """
    return await _run_mysql(spec, sql, read_only=True, max_rows=500)


async def db_row_count(
    spec: DbConnectionSpec, table: str, schema: str | None = None
) -> dict[str, Any]:
    t = _safe_ident(table)
    validate_connection(spec)
    if spec.engine == "sqlite":
        sql = f'SELECT COUNT(*) AS cnt FROM "{t}"'
        p = Path(spec.path or "")
        return await _run_sqlite(p, sql, read_only=True, max_rows=1)
    if spec.engine == "postgresql":
        sch = _safe_ident(schema or spec.schema or "public")
        sql = f'SELECT COUNT(*) AS cnt FROM "{sch}"."{t}"'
        return await _run_postgresql(spec, sql, read_only=True, max_rows=1)
    sql = f"SELECT COUNT(*) AS cnt FROM `{t}`"
    return await _run_mysql(spec, sql, read_only=True, max_rows=1)


async def db_inspect_operation(
    spec: DbConnectionSpec,
    operation: str,
    *,
    table: str | None = None,
    schema: str | None = None,
) -> dict[str, Any]:
    op = operation.strip().lower()
    if op == "ping":
        return await db_ping(spec)
    if op == "list_tables":
        return await db_list_tables(spec)
    if op == "list_schemas":
        return await db_list_schemas(spec)
    if op == "describe_table":
        if not table:
            return {"ok": False, "error": "describe_table requires table"}
        return await db_describe_table(spec, table, schema)
    if op in ("row_count", "table_row_count"):
        if not table:
            return {"ok": False, "error": "row_count requires table"}
        return await db_row_count(spec, table, schema)
    return {"ok": False, "error": f"unknown operation: {operation}"}



from __future__ import annotations

from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from aicd.agent.tool_router import ToolRouter


def _fs_list_dir(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    return router._fs.list_dir(args["path"])


def _fs_read_file(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    return router._fs.read_file(args["path"])


def _fs_write_file(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    return router._fs.write_file(args["path"], args["content"])


def _fs_delete(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    return router._fs.delete_path(args["path"], dry_run=bool(args.get("dry_run")))


def _fs_mkdir(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    return router._fs.mkdir(args["path"])


def _fs_move(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    return router._fs.move(args["from_path"], args["to_path"])


def _fs_glob(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    return router._fs.glob_names(args["directory"], args["pattern"])


def _fs_search_regex(router: ToolRouter, args: dict[str, Any]) -> dict[str, Any]:
    return router._fs.search_content_regex(
        args["directory"],
        args["regex"],
        glob_filter=str(args.get("glob_filter", "*")),
    )


HANDLERS: dict[str, Any] = {
    "fs_list_dir": _fs_list_dir,
    "fs_read_file": _fs_read_file,
    "fs_write_file": _fs_write_file,
    "fs_delete": _fs_delete,
    "fs_mkdir": _fs_mkdir,
    "fs_move": _fs_move,
    "fs_glob": _fs_glob,
    "fs_search_regex": _fs_search_regex,
}



from __future__ import annotations

import re
from pathlib import Path
from typing import Any

from aicd.tools.fs_tool import FSTool


class TextTool:
    def __init__(self, fs: FSTool) -> None:
        self._fs = fs

    def regex_replace(
        self, path: str, pattern: str, replacement: str, count: int = 0
    ) -> dict[str, Any]:
        r = self._fs.read_file(path)
        if not r.get("ok"):
            return r
        if r.get("truncated") or r.get("text") is None:
            return {"ok": False, "error": "file not utf-8 text or too large"}
        text = r["text"]
        assert isinstance(text, str)
        try:
            rx = re.compile(pattern)
        except re.error as e:
            return {"ok": False, "error": f"invalid regex: {e}"}
        new_text, n = rx.subn(replacement, text, count=count or 0)
        w = self._fs.write_file(path, new_text)
        if not w.get("ok"):
            return w
        return {"ok": True, "path": path, "replacements": n}

    def patch_bytes(
        self, path: str, offset: int, data_hex: str
    ) -> dict[str, Any]:
        try:
            patch = bytes.fromhex(data_hex.replace(" ", ""))
        except ValueError as e:
            return {"ok": False, "error": str(e)}
        r = self._fs.read_bytes_full(path)
        if not r.get("ok"):
            return r
        raw = bytearray(r["data"])
        assert isinstance(raw, bytearray)
        end = offset + len(patch)
        if offset < 0 or end > len(raw):
            return {"ok": False, "error": "offset/patch out of range"}
        raw[offset:end] = patch
        return self._fs.write_bytes(path, bytes(raw))



"""
跨平台桌面 GUI 辅助：枚举/聚焦窗口、截图、鼠标键盘（Windows + Linux X11 为主）。

Wayland 下鼠标键盘与截图能力取决于会话与是否安装 grim/wmctrl 等，见工具返回的 hint。
"""

from __future__ import annotations

import re
import subprocess
import sys
import time
from pathlib import Path
from typing import Any, Callable

MAX_DESKTOP_STEPS = 35
MAX_TYPE_CHARS = 8000

# PyAutoGUI 左上角触发异常退出；自动化时关闭以免误触
_PYAG_FAILSAFE = False


def _need_deps() -> str | None:
    try:
        import mss  # noqa: F401
        import pyautogui  # noqa: F401
    except ImportError as e:
        return (
            f"missing dependency: {e}; "
            "install: pip install aicd[desktop]  (or pip install pyautogui mss)"
        )
    return None


def _platform() -> str:
    if sys.platform == "win32":
        return "win32"
    return "linux"


# --- Windows: window list / focus / rect ---

def _win_list_windows(max_n: int = 80) -> list[dict[str, Any]]:
    import ctypes
    from ctypes import wintypes

    class RECT(ctypes.Structure):
        _fields_ = [
            ("left", ctypes.c_long),
            ("top", ctypes.c_long),
            ("right", ctypes.c_long),
            ("bottom", ctypes.c_long),
        ]

    user32 = ctypes.windll.user32
    out: list[dict[str, Any]] = []

    @ctypes.WINFUNCTYPE(ctypes.c_bool, wintypes.HWND, wintypes.LPARAM)
    def _cb(hwnd, _lparam):
        if not user32.IsWindowVisible(hwnd):
            return True
        ln = user32.GetWindowTextLengthW(hwnd)
        if ln <= 0:
            return True
        buf = ctypes.create_unicode_buffer(ln + 1)
        user32.GetWindowTextW(hwnd, buf, ln + 1)
        title = (buf.value or "").strip()
        if not title:
            return True
        r = RECT()
        user32.GetWindowRect(hwnd, ctypes.byref(r))
        out.append(
            {
                "window_id": str(int(hwnd)),
                "title": title[:500],
                "left": int(r.left),
                "top": int(r.top),
                "width": int(r.right - r.left),
                "height": int(r.bottom - r.top),
            }
        )
        return len(out) < max_n

    user32.EnumWindows(_cb, 0)
    return out


def _win_focus(hwnd: int) -> None:
    import ctypes
    from ctypes import wintypes

    user32 = ctypes.windll.user32
    SW_RESTORE = 9
    user32.ShowWindow(wintypes.HWND(hwnd), SW_RESTORE)
    user32.SetForegroundWindow(wintypes.HWND(hwnd))


def _win_hwnd_from_id(s: str) -> int:
    return int(str(s).strip(), 10)


# --- Linux: wmctrl / xdotool ---

def _linux_run(argv: list[str], timeout: float = 8.0) -> subprocess.CompletedProcess[str]:
    return subprocess.run(
        argv,
        capture_output=True,
        text=True,
        timeout=timeout,
        errors="replace",
    )


def _linux_list_windows_wmctrl(max_n: int) -> list[dict[str, Any]]:
    cr = _linux_run(["wmctrl", "-l", "-G"])
    if cr.returncode != 0 or not cr.stdout.strip():
        return []
    rows: list[dict[str, Any]] = []
    for line in cr.stdout.splitlines():
        # wid desk gx gy gw gh hostname title…
        m = re.match(
            r"^(0x[0-9a-fA-F]+)\s+(-?\d+)\s+(-?\d+)\s+(-?\d+)\s+(-?\d+)\s+(-?\d+)\s+(\S+)\s+(.*)$",
            line.strip(),
        )
        if not m:
            continue
        wid, _desk, gx, gy, gw, gh, _host, title = m.groups()
        rows.append(
            {
                "window_id": wid,
                "title": (title or "").strip()[:500],
                "left": int(gx),
                "top": int(gy),
                "width": int(gw),
                "height": int(gh),
            }
        )
        if len(rows) >= max_n:
            break
    if rows:
        return rows
    cr2 = _linux_run(["wmctrl", "-l"])
    if cr2.returncode != 0:
        return []
    for line in cr2.stdout.splitlines():
        parts = line.split(None, 3)
        if len(parts) < 4:
            continue
        rows.append(
            {
                "window_id": parts[0],
                "title": parts[3].strip()[:500],
                "left": 0,
                "top": 0,
                "width": 0,
                "height": 0,
                "geometry_note": "run wmctrl -l -G for x/y/w/h or use full-screen screenshot",
            }
        )
        if len(rows) >= max_n:
            break
    return rows


def _linux_focus_window_id(window_id: str) -> None:
    wid = window_id.strip()
    cr = _linux_run(["wmctrl", "-i", "-a", wid])
    if cr.returncode == 0:
        return
    # xdotool accepts decimal or hex with 0x
    xcr = _linux_run(["xdotool", "windowactivate", wid])
    if xcr.returncode != 0:
        raise RuntimeError(
            "focus_window failed (wmctrl -ia and xdotool windowactivate); "
            "install: apt install wmctrl xdotool (X11)"
        )


def list_windows(max_n: int = 80) -> dict[str, Any]:
    pf = _platform()
    if pf == "win32":
        return {"ok": True, "windows": _win_list_windows(max_n), "platform": pf}
    w = _linux_list_windows_wmctrl(max_n)
    if w:
        return {"ok": True, "windows": w, "platform": pf, "via": "wmctrl"}
    return {
        "ok": False,
        "error": "could not list windows (wmctrl -l failed or empty)",
        "hint": "Linux X11: install wmctrl (e.g. apt install wmctrl). Wayland may need compositor-specific tools.",
        "platform": pf,
        "windows": [],
    }


def focus_window(window_id: str) -> dict[str, Any]:
    pf = _platform()
    try:
        if pf == "win32":
            _win_focus(_win_hwnd_from_id(window_id))
        else:
            _linux_focus_window_id(window_id)
        return {"ok": True, "window_id": window_id}
    except Exception as e:  # noqa: BLE001
        return {"ok": False, "error": str(e)}


def _screenshot_region(
    path: Path,
    region: dict[str, Any] | None,
    policy_check: Callable[[Path], None],
) -> dict[str, Any]:
    import mss
    import mss.tools
    from PIL import Image

    policy_check(path)
    path.parent.mkdir(parents=True, exist_ok=True)
    if path.suffix.lower() not in {".png"}:
        return {"ok": False, "error": "screenshot path must be .png"}

    with mss.mss() as sct:
        if region:
            try:
                left = int(region["left"])
                top = int(region["top"])
                w = int(region["width"])
                h = int(region["height"])
            except (KeyError, TypeError, ValueError):
                return {"ok": False, "error": "region needs left, top, width, height (integers)"}
            if w < 1 or h < 1 or w > 16_384 or h > 16_384:
                return {"ok": False, "error": "region width/height out of range"}
            mon = {"left": left, "top": top, "width": w, "height": h}
        else:
            mon = sct.monitors[0]
        shot = sct.grab(mon)
        img = Image.frombytes("RGB", shot.size, shot.bgra, "raw", "BGRX")
        img.save(str(path), format="PNG")

    if not path.is_file() or path.stat().st_size == 0:
        return {"ok": False, "error": "PNG not written"}
    return {"ok": True, "png_path": str(path)}


def run_desktop_gui_automation(
    *,
    steps: list[dict[str, Any]],
    policy_check: Callable[[Path], None],
    cwd: Path,
) -> dict[str, Any]:
    dep = _need_deps()
    if dep:
        return {"ok": False, "error": dep}

    import pyautogui

    pyautogui.FAILSAFE = _PYAG_FAILSAFE

    if not isinstance(steps, list) or not steps:
        return {"ok": False, "error": "steps must be a non-empty list"}
    if len(steps) > MAX_DESKTOP_STEPS:
        return {"ok": False, "error": f"max {MAX_DESKTOP_STEPS} steps"}

    step_results: list[dict[str, Any]] = []
    pf = _platform()

    for i, raw in enumerate(steps):
        if not isinstance(raw, dict):
            return {
                "ok": False,
                "error": f"steps[{i}] must be object",
                "step_results": step_results,
                "failed_step_index": i,
            }
        op = str(raw.get("op", "")).strip().lower()
        rec: dict[str, Any] = {"index": i, "op": op}
        try:
            if op == "list_windows":
                mx = int(raw.get("max", 80))
                mx = max(1, min(200, mx))
                r = list_windows(mx)
                rec["ok"] = r.get("ok", False)
                rec["windows_n"] = len(r.get("windows") or [])
                rec["windows"] = (r.get("windows") or [])[:30]
                if not r.get("ok"):
                    rec["error"] = r.get("error")
                    rec["hint"] = r.get("hint")
                step_results.append(rec)
                if not r.get("ok"):
                    return {
                        "ok": False,
                        "error": r.get("error", "list_windows failed"),
                        "hint": r.get("hint"),
                        "step_results": step_results,
                        "failed_step_index": i,
                    }
                continue

            elif op == "focus_window":
                wid = str(raw.get("window_id", "")).strip()
                if not wid:
                    sub = str(raw.get("title_contains", "")).strip()
                    if not sub:
                        raise ValueError("focus_window needs window_id or title_contains")
                    lr = list_windows(120)
                    if not lr.get("ok"):
                        raise RuntimeError(lr.get("error", "list_windows failed"))
                    low = sub.lower()
                    match = None
                    for w in lr.get("windows") or []:
                        if low in (w.get("title") or "").lower():
                            match = w
                            break
                    if not match:
                        raise ValueError(f"no window title contains {sub!r}")
                    wid = str(match["window_id"])
                fr = focus_window(wid)
                if not fr.get("ok"):
                    raise RuntimeError(fr.get("error", "focus failed"))
                rec["ok"] = True
                rec["window_id"] = wid

            elif op == "screenshot":
                rel = str(raw.get("path", "")).strip()
                if not rel:
                    raise ValueError("screenshot needs path (.png)")
                p = Path(rel).expanduser()
                if not p.is_absolute():
                    p = (cwd / p).resolve()
                else:
                    p = p.resolve()
                reg = raw.get("region")
                reg_d = reg if isinstance(reg, dict) else None
                sr = _screenshot_region(p, reg_d, policy_check)
                if not sr.get("ok"):
                    raise RuntimeError(sr.get("error", "screenshot failed"))
                rec["ok"] = True
                rec["png_path"] = sr.get("png_path")

            elif op == "click":
                x = int(raw["x"])
                y = int(raw["y"])
                btn = str(raw.get("button", "left")).lower()
                clicks = int(raw.get("clicks", 1))
                clicks = max(1, min(3, clicks))
                interval = float(raw.get("click_interval_sec", 0.1))
                interval = max(0.0, min(2.0, interval))
                pyautogui.click(x=x, y=y, clicks=clicks, interval=interval, button=btn)
                rec["ok"] = True

            elif op == "double_click":
                x = int(raw["x"])
                y = int(raw["y"])
                btn = str(raw.get("button", "left")).lower()
                pyautogui.doubleClick(x=x, y=y, button=btn)
                rec["ok"] = True

            elif op == "move":
                x = int(raw["x"])
                y = int(raw["y"])
                dur = float(raw.get("duration_sec", 0.2))
                dur = max(0.0, min(10.0, dur))
                pyautogui.moveTo(x, y, duration=dur)
                rec["ok"] = True

            elif op == "drag":
                x = int(raw["x"])
                y = int(raw["y"])
                dur = float(raw.get("duration_sec", 0.3))
                dur = max(0.05, min(30.0, dur))
                pyautogui.dragTo(x, y, duration=dur, button=str(raw.get("button", "left")))
                rec["ok"] = True

            elif op == "scroll":
                clicks = int(raw.get("clicks", -3))
                clicks = max(-500, min(500, clicks))
                pyautogui.scroll(clicks)
                rec["ok"] = True

            elif op == "type_text":
                text = str(raw.get("text", ""))
                if len(text) > MAX_TYPE_CHARS:
                    raise ValueError(f"text too long (max {MAX_TYPE_CHARS})")
                interval = float(raw.get("interval_sec", 0.0))
                interval = max(0.0, min(1.0, interval))
                pyautogui.write(text, interval=interval)
                rec["ok"] = True

            elif op == "press":
                key = str(raw.get("key", "")).strip()
                if not key:
                    raise ValueError("press needs key")
                presses = int(raw.get("presses", 1))
                presses = max(1, min(20, presses))
                interval = float(raw.get("interval_sec", 0.1))
                pyautogui.press(key, presses=presses, interval=interval)
                rec["ok"] = True

            elif op == "hotkey":
                keys = raw.get("keys")
                if not isinstance(keys, list) or not keys:
                    raise ValueError("hotkey needs keys: string array")
                ks = [str(k).strip() for k in keys if str(k).strip()]
                if not ks or len(ks) > 6:
                    raise ValueError("hotkey keys: 1–6 non-empty strings")
                pyautogui.hotkey(*ks)
                rec["ok"] = True

            elif op == "wait_ms":
                ms = int(raw.get("ms", 0))
                ms = max(0, min(120_000, ms))
                time.sleep(ms / 1000.0)
                rec["ok"] = True

            elif op == "pixel":
                x = int(raw["x"])
                y = int(raw["y"])
                im = pyautogui.screenshot(region=(x, y, 1, 1))
                rgb = im.getpixel((0, 0))
                rec["ok"] = True
                rec["rgb"] = rgb

            else:
                raise ValueError(
                    f"unknown op {op!r}; use list_windows, focus_window, screenshot, "
                    "click, double_click, move, drag, scroll, type_text, press, hotkey, wait_ms, pixel"
                )

            step_results.append(rec)

        except Exception as e:  # noqa: BLE001
            rec["ok"] = False
            rec["error"] = str(e)
            step_results.append(rec)
            return {
                "ok": False,
                "error": f"step {i} ({op}): {e}",
                "failed_step_index": i,
                "step_results": step_results,
                "platform": pf,
            }

    return {
        "ok": True,
        "step_results": step_results,
        "platform": pf,
        "note": "Coordinates are screen pixels. On Linux use X11 + wmctrl/xdotool; Wayland support varies. "
        "Combine screenshot png with vision_analyze_image for visual checks.",
    }



"""从 Markdown / HTML 提取标题层级与行号，供大文档分块阅读。"""

from __future__ import annotations

import json
import re
from pathlib import Path
from typing import Any

# ATX 风格 Markdown 标题
_MD_HEADING = re.compile(r"^(#{1,6})\s+(.+?)\s*$")
# 单行内的 HTML 标题（逐行扫描）
_HTML_HEADING = re.compile(
    r"<h\s*([1-6])[^>]*>(.*?)</h\s*\1\s*>",
    re.IGNORECASE | re.DOTALL,
)


def extract_outline_from_text(
    text: str,
    *,
    source_path: str,
    max_heading_level: int = 6,
    format_hint: str | None = None,
) -> list[dict[str, Any]]:
    lines = text.splitlines()
    fmt = (format_hint or "").strip().lower()
    is_html = fmt == "html" or (not fmt and _looks_like_html(text[:2000]))

    out: list[dict[str, Any]] = []
    if is_html:
        for i, line in enumerate(lines, start=1):
            for m in _HTML_HEADING.finditer(line):
                level = int(m.group(1))
                if level > max_heading_level:
                    continue
                raw = m.group(2)
                title = re.sub(r"<[^>]+>", "", raw).strip()
                if title:
                    byte_off = _byte_offset_at_line_start(text, i)
                    out.append(
                        {
                            "level": level,
                            "line": i,
                            "text": title[:500],
                            "byte_offset": byte_off,
                        }
                    )
        return out

    for i, line in enumerate(lines, start=1):
        m = _MD_HEADING.match(line)
        if not m:
            continue
        level = len(m.group(1))
        if level > max_heading_level:
            continue
        title = m.group(2).strip()
        byte_off = _byte_offset_at_line_start(text, i)
        out.append(
            {
                "level": level,
                "line": i,
                "text": title[:500],
                "byte_offset": byte_off,
            }
        )
    return out


def _looks_like_html(sample: str) -> bool:
    s = sample.lower()
    return "<h1" in s or "<h2" in s or "<html" in s or "<body" in s


def _byte_offset_at_line_start(full_text: str, line_no: int) -> int:
    """1-based 行号 → UTF-8 字节偏移（该行行首）。"""
    if line_no <= 1:
        return 0
    parts = full_text.splitlines(keepends=True)
    if line_no > len(parts):
        return len(full_text.encode("utf-8"))
    acc = 0
    for j in range(line_no - 1):
        acc += len(parts[j].encode("utf-8"))
    return acc


def build_outline_payload(
    path: Path,
    *,
    max_heading_level: int = 6,
) -> dict[str, Any]:
    raw = path.read_text(encoding="utf-8", errors="replace")
    suf = path.suffix.lower()
    hint = "html" if suf in (".html", ".htm") else ("markdown" if suf in (".md", ".markdown") else None)
    headings = extract_outline_from_text(
        raw,
        source_path=str(path),
        max_heading_level=max_heading_level,
        format_hint=hint,
    )
    return {
        "source_path": str(path.resolve()),
        "heading_count": len(headings),
        "headings": headings,
        "format_detected": hint or ("html" if _looks_like_html(raw[:2000]) else "markdown"),
    }


def write_outline_json(payload: dict[str, Any], dest: Path) -> None:
    dest.parent.mkdir(parents=True, exist_ok=True)
    dest.write_text(
        json.dumps(payload, ensure_ascii=False, indent=2),
        encoding="utf-8",
    )



"""运行时合并 agent/llm 配置补丁（斜杠命令与 agent_settings_update 工具）。"""

from __future__ import annotations

import copy
from typing import Any

from aicd.config import MAX_AGENT_TOOL_ROUNDS, AppConfig, LLMConfig


def _clamp_int(v: int, lo: int, hi: int) -> int:
    return max(lo, min(hi, v))


def _clamp_agent_heartbeat(sec: int) -> int:
    """0=关闭；正数夹紧到 [5, 3600]（秒）。"""
    if sec <= 0:
        return 0
    return max(5, min(3600, int(sec)))


def _parse_float(raw: str) -> float | None:
    try:
        return float(raw.strip())
    except (TypeError, ValueError):
        return None


def _parse_int(raw: str) -> int | None:
    try:
        return int(raw.strip(), 10)
    except (TypeError, ValueError):
        return None


def set_llm_scalar_from_slash(cfg: LLMConfig, key: str, value_str: str) -> tuple[bool, str]:
    """``/llm <key> <value>`` 解析；``key`` 已小写。"""
    k = key.strip().lower().replace("-", "_")
    aliases = {
        "temp": "temperature",
        "topp": "top_p",
        "top_p": "top_p",
        "freq": "frequency_penalty",
        "frequencypenalty": "frequency_penalty",
        "frequency_penalty": "frequency_penalty",
        "pres": "presence_penalty",
        "presencepenalty": "presence_penalty",
        "presence_penalty": "presence_penalty",
        "maxtokens": "max_output_tokens",
        "max_output_tokens": "max_output_tokens",
        "maxoutputtokens": "max_output_tokens",
        "repetition": "repetition_penalty",
        "repetitionpenalty": "repetition_penalty",
        "repeat_penalty": "repetition_penalty",
        "repetition_penalty": "repetition_penalty",
        "timeout": "timeout_sec",
        "timeoutsec": "timeout_sec",
        "model": "model",
        "modelid": "model",
        "maxretries": "max_retries",
        "retrydelay": "retry_base_delay_sec",
        "retrybasedelaysec": "retry_base_delay_sec",
    }
    field = aliases.get(k, k)
    if field == "model":
        cfg.model = value_str.strip()
        return True, f"llm.model={cfg.model!r}"
    if field in ("max_retries", "maxretries", "retries"):
        n = _parse_int(value_str)
        if n is None or n < 0:
            return False, "max_retries must be a non-negative integer (0–20)"
        cfg.max_retries = _clamp_int(n, 0, 20)
        return True, f"llm.max_retries={cfg.max_retries}"
    if field in ("retry_base_delay_sec", "retrybasedelay", "retry_delay"):
        f = _parse_float(value_str)
        if f is None or f <= 0:
            return False, "retry_base_delay_sec must be > 0"
        cfg.retry_base_delay_sec = float(f)
        return True, f"llm.retry_base_delay_sec={cfg.retry_base_delay_sec}"
    if field == "timeout_sec":
        f = _parse_float(value_str)
        if f is None or f <= 0:
            return False, "timeout_sec must be a positive number"
        cfg.timeout_sec = float(f)
        return True, f"llm.timeout_sec={cfg.timeout_sec}"
    if field == "temperature":
        f = _parse_float(value_str)
        if f is None or not 0 <= f <= 2:
            return False, "temperature must be a number in [0, 2]"
        cfg.temperature = float(f)
        return True, f"llm.temperature={cfg.temperature}"
    if field == "top_p":
        f = _parse_float(value_str)
        if f is None or not 0 < f <= 1:
            return False, "top_p must be in (0, 1]"
        cfg.top_p = float(f)
        return True, f"llm.top_p={cfg.top_p}"
    if field == "frequency_penalty":
        f = _parse_float(value_str)
        if f is None or not -2 <= f <= 2:
            return False, "frequency_penalty must be in [-2, 2]"
        cfg.frequency_penalty = float(f)
        return True, f"llm.frequency_penalty={cfg.frequency_penalty}"
    if field == "presence_penalty":
        f = _parse_float(value_str)
        if f is None or not -2 <= f <= 2:
            return False, "presence_penalty must be in [-2, 2]"
        cfg.presence_penalty = float(f)
        return True, f"llm.presence_penalty={cfg.presence_penalty}"
    if field == "max_output_tokens":
        n = _parse_int(value_str)
        if n is None or n < 1:
            return False, "max_output_tokens must be a positive integer"
        cfg.max_output_tokens = int(n)
        return True, f"llm.max_output_tokens={cfg.max_output_tokens}"
    if field == "repetition_penalty":
        f = _parse_float(value_str)
        if f is None or f <= 0:
            return False, "repetition_penalty must be a positive number"
        cfg.repetition_penalty = float(f)
        return True, f"llm.repetition_penalty={cfg.repetition_penalty}"
    return False, f"unknown llm key: {key!r} (try temperature, topP, maxOutputTokens, …)"


def set_agent_scalar_from_slash(cfg: AppConfig, key: str, value_str: str) -> tuple[bool, str]:
    k = key.strip().lower().replace("-", "_")
    n = _parse_int(value_str)
    if n is None:
        return False, "value must be an integer"
    if k in ("rounds", "max_tool_rounds", "maxtoolrounds"):
        cfg.agent.max_tool_rounds = _clamp_int(n, 1, MAX_AGENT_TOOL_ROUNDS)
        return True, f"agent.max_tool_rounds={cfg.agent.max_tool_rounds}"
    if k in ("subrounds", "sub_rounds", "subagent", "sub_agent_max_tool_rounds", "subagentmaxtoolrounds"):
        cfg.agent.sub_agent_max_tool_rounds = _clamp_int(n, 1, MAX_AGENT_TOOL_ROUNDS)
        return True, f"agent.sub_agent_max_tool_rounds={cfg.agent.sub_agent_max_tool_rounds}"
    if k in ("heartbeat", "idle_heartbeat", "idleheartbeat", "idle_heartbeat_sec"):
        cfg.agent.idle_heartbeat_sec = _clamp_agent_heartbeat(n)
        return True, f"agent.idle_heartbeat_sec={cfg.agent.idle_heartbeat_sec} (0=关闭, 否则 5–3600 秒)"
    if k in (
        "subheartbeat",
        "sub_heartbeat",
        "sub_agent_idle_heartbeat",
        "subagentidleheartbeat",
        "sub_agent_idle_heartbeat_sec",
    ):
        cfg.agent.sub_agent_idle_heartbeat_sec = _clamp_agent_heartbeat(n)
        return (
            True,
            f"agent.sub_agent_idle_heartbeat_sec={cfg.agent.sub_agent_idle_heartbeat_sec} "
            "(0=关闭；子任务仍有 running/pending 时启用协调心跳)",
        )
    return False, f"unknown agent key: {key!r} (rounds | subRounds | heartbeat | subHeartbeat)"


def apply_nested_settings_patch(cfg: AppConfig, patch: dict[str, Any]) -> tuple[bool, str]:
    """
    ``patch`` 形如 ``{"llm": {"temperature": 0.5}, "agent": {"maxToolRounds": 30}}``。
    仅更新出现的字段；校验失败则不改配置并返回错误说明。
    """
    if not isinstance(patch, dict):
        return False, "patch must be an object"
    errs: list[str] = []
    backup = copy.deepcopy(cfg)

    try:
        llm_p = patch.get("llm")
        if isinstance(llm_p, dict):
            for rk, rv in llm_p.items():
                key = str(rk).replace("-", "_")
                camel_map = {
                    "baseUrl": "base_url",
                    "apiKey": "api_key",
                    "modelId": "model",
                    "timeoutSec": "timeout_sec",
                    "topP": "top_p",
                    "frequencyPenalty": "frequency_penalty",
                    "presencePenalty": "presence_penalty",
                    "maxOutputTokens": "max_output_tokens",
                    "repetitionPenalty": "repetition_penalty",
                    "extraBody": "extra_body",
                    "visionModel": "vision_model",
                    "maxRetries": "max_retries",
                    "retryBaseDelaySec": "retry_base_delay_sec",
                }
                key = camel_map.get(key, key)
                if key == "model":
                    cfg.llm.model = str(rv).strip()
                elif key == "vision_model":
                    cfg.llm.vision_model = (
                        str(rv).strip() if str(rv).strip() else None
                    )
                elif key == "proto":
                    cfg.llm.proto = str(rv).strip()
                elif key == "base_url":
                    cfg.llm.base_url = str(rv).strip()
                elif key == "api_key":
                    cfg.llm.api_key = str(rv).strip()
                elif key == "timeout_sec":
                    cfg.llm.timeout_sec = float(rv)
                    if cfg.llm.timeout_sec <= 0:
                        raise ValueError("timeout_sec must be > 0")
                elif key == "max_retries":
                    cfg.llm.max_retries = _clamp_int(int(rv), 0, 20)
                elif key == "retry_base_delay_sec":
                    cfg.llm.retry_base_delay_sec = max(0.1, float(rv))
                elif key == "temperature":
                    cfg.llm.temperature = float(rv)
                    if not 0 <= cfg.llm.temperature <= 2:
                        raise ValueError("temperature in [0,2]")
                elif key == "top_p":
                    cfg.llm.top_p = float(rv)
                    if not 0 < cfg.llm.top_p <= 1:
                        raise ValueError("top_p in (0,1]")
                elif key == "frequency_penalty":
                    cfg.llm.frequency_penalty = float(rv)
                    if not -2 <= cfg.llm.frequency_penalty <= 2:
                        raise ValueError("frequency_penalty in [-2,2]")
                elif key == "presence_penalty":
                    cfg.llm.presence_penalty = float(rv)
                    if not -2 <= cfg.llm.presence_penalty <= 2:
                        raise ValueError("presence_penalty in [-2,2]")
                elif key == "max_output_tokens":
                    cfg.llm.max_output_tokens = int(rv)
                    if cfg.llm.max_output_tokens < 1:
                        raise ValueError("max_output_tokens >= 1")
                elif key == "repetition_penalty":
                    cfg.llm.repetition_penalty = float(rv)
                    if cfg.llm.repetition_penalty <= 0:
                        raise ValueError("repetition_penalty > 0")
                elif key == "extra_body":
                    if rv is not None and not isinstance(rv, dict):
                        raise ValueError("extra_body must be an object")
                    cfg.llm.extra_body = dict(rv) if rv else {}
                elif key == "clear_sampling":
                    if rv:
                        cfg.llm.temperature = None
                        cfg.llm.top_p = None
                        cfg.llm.frequency_penalty = None
                        cfg.llm.presence_penalty = None
                        cfg.llm.max_output_tokens = None
                        cfg.llm.repetition_penalty = None
                else:
                    errs.append(f"unknown llm field: {rk!r}")

        ag_p = patch.get("agent")
        if isinstance(ag_p, dict):
            for rk, rv in ag_p.items():
                k = str(rk)
                if k in ("maxToolRounds", "max_tool_rounds"):
                    cfg.agent.max_tool_rounds = _clamp_int(
                        int(rv), 1, MAX_AGENT_TOOL_ROUNDS
                    )
                elif k in ("subAgentMaxToolRounds", "sub_agent_max_tool_rounds"):
                    cfg.agent.sub_agent_max_tool_rounds = _clamp_int(
                        int(rv), 1, MAX_AGENT_TOOL_ROUNDS
                    )
                elif k in ("contextBudgetTokens", "context_budget_tokens"):
                    cfg.agent.context_budget_tokens = max(4096, int(rv))
                elif k in ("chatHistoryMaxMessages", "chat_history_max_messages"):
                    cfg.agent.chat_history_max_messages = max(
                        10, min(20_000, int(rv))
                    )
                elif k in ("idleHeartbeatSec", "idle_heartbeat_sec"):
                    cfg.agent.idle_heartbeat_sec = _clamp_agent_heartbeat(int(rv))
                elif k in (
                    "subAgentIdleHeartbeatSec",
                    "sub_agent_idle_heartbeat_sec",
                ):
                    cfg.agent.sub_agent_idle_heartbeat_sec = _clamp_agent_heartbeat(
                        int(rv)
                    )
                else:
                    errs.append(f"unknown agent field: {rk!r}")
    except (TypeError, ValueError) as e:
        # restore
        cfg.llm = backup.llm
        cfg.agent = backup.agent
        return False, str(e)

    if errs:
        cfg.llm = backup.llm
        cfg.agent = backup.agent
        return False, "; ".join(errs)
    return True, "ok"


def format_settings_summary(cfg: AppConfig) -> str:
    llm = cfg.llm
    lines = [
        "[agent]",
        f"  max_tool_rounds={cfg.agent.max_tool_rounds}",
        f"  sub_agent_max_tool_rounds={cfg.agent.sub_agent_max_tool_rounds}",
        f"  idle_heartbeat_sec={cfg.agent.idle_heartbeat_sec} (主对话空闲定时检查；0=关)",
        f"  sub_agent_idle_heartbeat_sec={cfg.agent.sub_agent_idle_heartbeat_sec} (子 Agent 协调；0=关)",
        f"  context_budget_tokens={cfg.agent.context_budget_tokens}",
        f"  chat_history_max_messages={cfg.agent.chat_history_max_messages}",
        "[llm sampling]",
        f"  temperature={llm.temperature}",
        f"  top_p={llm.top_p}",
        f"  frequency_penalty={llm.frequency_penalty}",
        f"  presence_penalty={llm.presence_penalty}",
        f"  max_output_tokens={llm.max_output_tokens}",
        f"  repetition_penalty={llm.repetition_penalty}",
        f"  extra_body keys={list(llm.extra_body.keys()) if llm.extra_body else []}",
        "[llm core]",
        f"  model={llm.model!r} vision_model={llm.vision_model!r} timeout_sec={llm.timeout_sec}",
        f"  max_retries={llm.max_retries} retry_base_delay_sec={llm.retry_base_delay_sec}",
    ]
    return "\n".join(lines)



from __future__ import annotations

from aicd.kit import chart_spec


def test_series_to_chartjs_config() -> None:
    cfg = chart_spec.series_to_chartjs_config("bar", ["a", "b"], [1, 2])
    assert cfg["type"] == "bar"
    assert cfg["data"]["labels"] == ["a", "b"]
    assert cfg["data"]["datasets"][0]["data"] == [1.0, 2.0]


def test_line_has_fill_false() -> None:
    cfg = chart_spec.series_to_chartjs_config("line", ["a"], [1])
    assert cfg["type"] == "line"
    assert cfg["data"]["datasets"][0]["fill"] is False


def test_mermaid_bar_text() -> None:
    s = chart_spec.mermaid_bar_from_counts(["x", "y"], [1, 4])
    assert "x" in s
    assert "####" in s or "#" in s


def test_build_html_with_chart() -> None:
    html = chart_spec.build_html_with_chart("T", "bar", ["p"], [3.0])
    assert "Chart" in html
    assert "chart.umd.min.js" in html



"""按像素矩形裁剪栅格图（输出 PNG）。"""

from __future__ import annotations

from pathlib import Path
from typing import Any, Callable


def crop_raster_image(
    input_path: Path,
    output_png: Path,
    *,
    left: int,
    top: int,
    width: int,
    height: int,
    policy_check: Callable[[Path], None],
    bbox_reference_width: int | None = None,
    bbox_reference_height: int | None = None,
) -> dict[str, Any]:
    """
    ``left/top/width/height`` 默认相对于 **input_path** 的像素。

    若 bbox 来自 **vision_analyze_image** 等在 **缩小后的图**（``sent_width``×``sent_height``）上描述的坐标，
    设 ``bbox_reference_width`` = ``sent_width``、``bbox_reference_height`` = ``sent_height``，
    则会按比例映射到 **原图** 再裁剪（插入 Word 前应使用原图路径 + 此参数，勿对缩小图裁剪后当终稿）。
    """
    from PIL import Image

    inp = input_path.expanduser().resolve()
    outp = output_png.expanduser().resolve()
    policy_check(inp)
    policy_check(outp)
    if not inp.is_file():
        return {"ok": False, "error": f"input not found: {inp}"}
    if outp.suffix.lower() != ".png":
        return {"ok": False, "error": "output_path must end with .png"}
    if width <= 0 or height <= 0:
        return {"ok": False, "error": "width and height must be positive"}

    with Image.open(inp) as im:
        W, H = im.size
        rw = bbox_reference_width
        rh = bbox_reference_height
        if rw is not None and rh is not None and rw > 0 and rh > 0:
            sx = W / float(rw)
            sy = H / float(rh)
            left = int(round(left * sx))
            top = int(round(top * sy))
            width = max(1, int(round(width * sx)))
            height = max(1, int(round(height * sy)))
        left = max(0, min(left, max(0, W - 1)))
        top = max(0, min(top, max(0, H - 1)))
        width = min(width, W - left)
        height = min(height, H - top)
        if width <= 0 or height <= 0:
            return {
                "ok": False,
                "error": f"crop box empty after clamp: image {W}x{H}",
            }
        box = (left, top, left + width, top + height)
        cropped = im.crop(box)
        outp.parent.mkdir(parents=True, exist_ok=True)
        cropped.save(outp, format="PNG")

    return {
        "ok": True,
        "path": str(outp),
        "source": str(inp),
        "crop": {"left": left, "top": top, "width": width, "height": height},
    }



"""CLI 行为烟测（不启动 TUI / 不调 LLM）。"""

from typer.testing import CliRunner

from aicd.app import app


def test_tui_noui_requires_ai() -> None:
    runner = CliRunner()
    r = runner.invoke(app, ["tui", "--noui"])
    assert r.exit_code == 2
    assert "--ai" in (r.stdout or "") + (r.stderr or "")



"""script_run 命名与 aicd/bak 归档。"""

from __future__ import annotations

from datetime import datetime, timezone
from pathlib import Path

from aicd.tools.script_backup import (
    archive_script_to_bak,
    format_script_run_filename,
    policy_allow_under_ai_output_or_workspace,
    sanitize_doc_label,
)


def test_sanitize_doc_label() -> None:
    assert sanitize_doc_label(None) == "adhoc"
    assert sanitize_doc_label("  ") == "adhoc"
    assert "manual" in sanitize_doc_label("output/library/foo/manual_v2.md")
    assert "_" in sanitize_doc_label("a/b\\c")


def test_format_script_run_filename_stable_shape() -> None:
    ts = datetime(2025, 3, 23, 8, 15, 30, tzinfo=timezone.utc)
    fn = format_script_run_filename(ts, "py", ".py", "spec_v1", "a1b2c3d4")
    assert fn.startswith("20250323T081530Z_py_spec_v1_")
    assert fn.endswith(".py")


def test_policy_allow_under_ai_output(tmp_path: Path) -> None:
    ai_out = tmp_path / "proj" / "aicd"
    outside = tmp_path / "other" / "x.txt"

    def _deny(_p: Path) -> None:
        raise PermissionError("blocked")

    pol = policy_allow_under_ai_output_or_workspace(ai_out, _deny)
    pol((ai_out / "bak" / "s.py").resolve())  # does not raise
    try:
        pol(outside)
    except PermissionError as e:
        assert "blocked" in str(e)
    else:
        raise AssertionError("expected PermissionError")


def test_archive_script_to_bak_copies_and_removes_tmp(tmp_path: Path) -> None:
    bak = tmp_path / "aicd" / "bak"
    src = tmp_path / "20250323T081530Z_py_adhoc_x1.py"
    src.write_text("print(1)", encoding="utf-8")

    def _ok(p: Path) -> None:
        return None

    r = archive_script_to_bak(src, bak, policy_check=_ok)
    assert r.get("script_backup_ok") is True
    assert r.get("script_bak_path")
    dest = Path(r["script_bak_path"])
    assert dest.is_file()
    assert dest.read_text(encoding="utf-8") == "print(1)"
    assert not src.exists()



"""Windows CMD: python -c 与 && 链、引号预检。"""

from __future__ import annotations

import sys

from aicd.tools.process_tool import (
    warn_shell_python_c_unbalanced_quotes,
    warn_shell_risky_python_c,
    warn_shell_windows_findstr_maybe_dir_operand,
    warn_shell_windows_python_c_with_and_chain,
)


def test_unbalanced_quotes_detected() -> None:
    msg = warn_shell_python_c_unbalanced_quotes('python -c "import os')
    assert msg is not None
    assert "双引号" in msg or "未成对" in msg


def test_balanced_quotes_ok() -> None:
    assert (
        warn_shell_python_c_unbalanced_quotes(
            'python -c "import os; print(1)"'
        )
        is None
    )


def test_and_chain_python_c_on_windows_only() -> None:
    cmd = 'cd D:\\x && python -c "import os; print(1)"'
    if sys.platform == "win32":
        msg = warn_shell_windows_python_c_with_and_chain(cmd)
        assert msg is not None
        assert "&&" in msg or "同一行" in msg
        assert warn_shell_risky_python_c(cmd) == msg
    else:
        assert warn_shell_windows_python_c_with_and_chain(cmd) is None


def test_findstr_warns_when_last_operand_looks_like_dir_name() -> None:
    cmd = 'findstr /s /i "class" regional-control'
    if sys.platform == "win32":
        msg = warn_shell_windows_findstr_maybe_dir_operand(cmd)
        assert msg is not None
        assert "findstr" in msg.lower() or "文件夹" in msg or "Cannot" in msg
    else:
        assert warn_shell_windows_findstr_maybe_dir_operand(cmd) is None


def test_findstr_no_warn_with_glob() -> None:
    cmd = r'findstr /s /i "class" regional-control\*.*'
    if sys.platform == "win32":
        assert warn_shell_windows_findstr_maybe_dir_operand(cmd) is None
